// 🐦 Flutter imports:
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

// 📦 Package imports:
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import 'package:verttice_ui/src/widgets/data_factory/common/form_item_to_filter_item.dart';
import 'package:verttice_ui/verttice_ui.dart';

class VisualData<T> {
  final Widget widget;
  final T value;
  Type type = T;

  VisualData({
    required this.widget,
    required this.value,
  });
}

VisualData<dynamic> visualDataProcessor({
  required BuildContext context,
  required FormItem item,
  required Map<String, dynamic> row,
  bool isFilter = false,
}) {
  final formItem = isFilter ? formItemToFilterItem(formItem: item) : item;
  dynamic value = row[formItem.field.name];

  if (formItem.field.dataBuilder != null) {
    value = formItem.field.dataBuilder!(row);
  }

  dynamic returnValue;
  Widget? widget;
  if (formItem.field is ColorSelectorFF) {
    SelectorColors colorSelectorValue = SelectorColors.grey;
    for (var selectorColor in SelectorColors.values) {
      if (selectorColor.name == value) {
        colorSelectorValue = selectorColor;
      }
    }
    late Color color;
    switch (colorSelectorValue) {
      case SelectorColors.green:
        color = Colors.green;
        break;
      case SelectorColors.red:
        color = Colors.red;
        break;

      case SelectorColors.orange:
        color = Colors.orange;
        break;

      case SelectorColors.grey:
        color = Colors.grey;
        break;

      case SelectorColors.yellow:
        color = Colors.yellow;
        break;
    }

    return VisualData(
      value: colorSelectorValue.name,
      widget: Container(
        color: color,
      ),
    );
  } else if (formItem.field is FilesFF) {
    returnValue = "Files 📂";
  } else if (formItem.field is ImageFF) {
    returnValue = "Image 🖼️";
  } else if (formItem.field is IntegerFF ||
      formItem.field is DoubleFF ||
      formItem.field is ReadonlyNumberFF) {
    returnValue = double.parse(DataClassUtils.dynamicToDouble(
      value,
      defaultValue: 0,
    )!
        .toStringAsFixed(2));
  } else if (formItem.field is DateFF || formItem.field is ReadonlyDateFF) {
    returnValue = value == null || value is! int
        ? ""
        : Dates.formatDataAndTime(
            milliseconds: DataClassUtils.dynamicToInt(value, defaultValue: 0)!,
            date: true,
            time: false,
          );
  } else if (formItem.field is TimeFF || formItem.field is ReadonlyTimeFF) {
    returnValue = value == null || value is! int
        ? ""
        : Dates.formatDataAndTime(
            milliseconds: DataClassUtils.dynamicToInt(value, defaultValue: 0)!,
            date: false,
            time: true,
          );
  } else if (formItem.field is ReadonlyDateTimeFF) {
    returnValue = value == null || value is! int
        ? ""
        : Dates.formatDataAndTime(
            milliseconds: DataClassUtils.dynamicToInt(value, defaultValue: 0)!,
            date: true,
            time: true,
          );
  } else if (formItem.field is NumberRangeFF ||
      formItem.field is ReadonlyNumberRangeFF) {
    final numberRange = visualNumberRangeModel(value, context: context);
    returnValue = numberRange.value;
    widget = numberRange.widget;
  } else if (formItem.field is DateRangeFF ||
      formItem.field is ReadonlyDateRangeFF) {
    final dateRange = visualDateRangeModel(value, context: context);

    returnValue = dateRange.value;
    widget = dateRange.widget;
  } else if (formItem.field is DateTimeRangeFF ||
      formItem.field is ReadonlyDateTimeRangeFF) {
    returnValue = value == null || value is! Map
        ? ""
        : "${value["startDate"] != null && value["startDate"] is int ? Dates.formatDataAndTime(milliseconds: DataClassUtils.dynamicToInt(value["startDate"], defaultValue: 0)!) : ""} - \n${value["endDate"] != null && value["endDate"] is int ? Dates.formatDataAndTime(milliseconds: DataClassUtils.dynamicToInt(value["endDate"], defaultValue: 0)!) : ""}";
  } else if (formItem.field is PhoneNumberFF) {
    final phone = visualPhoneNumber(value, context: context);
    returnValue = phone.value;
    widget = phone.widget;
  } else if (formItem.field is PhoneNumberListFF) {
    final phones = visualPhoneNumberList(value, context: context);
    returnValue = phones.value;
    widget = phones.widget;
  } else if (formItem.field is AddressOnMapFF) {
    final address = visualAddressOnMap(value, context: context);
    returnValue = address.value;
    widget = address.widget;
  } else if (formItem.field is EmailListFF || formItem.field is TextListFF) {
    returnValue = value is List ? value.join("\n") : "";
  } else {
    returnValue = DataClassUtils.dynamicToString(value);
  }

  return VisualData(
    value: returnValue,
    widget: widget ??
        InkWell(
          onTap: () {
            Clipboard.setData(
              ClipboardData(
                text: returnValue,
              ),
            );
            ThemedToast.info(
              context: context,
              message: "Copied to clipboard",
            );
          },
          child: Text(
            returnValue?.toString() ?? "",
            style: const TextStyle(
              fontSize: 12,
            ),
          ),
        ),
  );
}

VisualData<String> visualAddressOnMap(
  dynamic value, {
  required BuildContext context,
}) {
  final address = AddressOnMapModel.fromMap(
    DataClassUtils.dynamicToMapStringDynamic(value),
  );
  final isLatLngExist =
      address.latLng?.lat != null && address.latLng?.lng != null;
  final copyValue = address.fullAddress ?? "";
  final returnValue = address.fullAddress ?? "";
  final controller = ScrollController();

  return VisualData(
      value: returnValue,
      widget: Tooltip(
        message: copyValue,
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            if (isLatLngExist)
              SizedBox(
                width: 13,
                child: Center(
                  child: Tooltip(
                    message: 'Open address on google maps',
                    child: InkWell(
                      onTap: () async {
                        final url =
                            "https://www.google.com/maps/search/?api=1&query=${address.latLng?.lat},${address.latLng?.lng}";
                        if (await canLaunchUrl(Uri.parse(url))) {
                          await launchUrl(Uri.parse(url));
                        }
                      },
                      child: const Text(
                        '📍',
                      ),
                    ),
                  ),
                ),
              ),
            Expanded(
              child: Padding(
                padding: const EdgeInsets.only(left: 2),
                child: Scrollbar(
                  scrollbarOrientation: ScrollbarOrientation.right,
                  thumbVisibility: true,
                  controller: controller,
                  child: SingleChildScrollView(
                    controller: controller,
                    scrollDirection: Axis.vertical,
                    child: InkWell(
                      onTap: () async {
                        Clipboard.setData(
                          ClipboardData(
                            text: copyValue,
                          ),
                        );
                        ThemedToast.info(
                          context: context,
                          message: "Address copied to clipboard",
                        );
                      },
                      child: Text(
                        returnValue,
                        overflow: TextOverflow.fade,
                        softWrap: true,
                        maxLines: 10,
                        style: const TextStyle(
                          fontSize: 11.5,
                        ),
                      ),
                    ),
                  ),
                ),
              ),
            )
          ],
        ),
      ));
}

VisualData<String> visualNumberRangeModel(
  dynamic value, {
  required BuildContext context,
}) {
  final returnValue = value == null || value is! Map
      ? ""
      : "${value["lowNumber"] != null && value["lowNumber"] is int ? value["lowNumber"].toString() : ""} - ${value["heighNumber"] != null && value["heighNumber"] is int ? value["heighNumber"].toString() : ""}";

  return VisualData(
    value: returnValue,
    widget: InkWell(
      onTap: () {
        Clipboard.setData(
          ClipboardData(
            text: returnValue,
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Number range copied to clipboard",
        );
      },
      child: Text(
        returnValue,
        style: const TextStyle(
          fontSize: 12,
        ),
      ),
    ),
  );
}

VisualData<String> visualDateRangeModel(
  dynamic value, {
  required BuildContext context,
}) {
  final returnValue = value == null || value is! Map
      ? ""
      : "${value["startDate"] != null && value["startDate"] is int ? Dates.formatDataAndTime(milliseconds: DataClassUtils.dynamicToInt(value["startDate"], defaultValue: 0)!, time: false) : ""} - ${value["endDate"] != null && value["endDate"] is int ? Dates.formatDataAndTime(milliseconds: DataClassUtils.dynamicToInt(value["endDate"], defaultValue: 0)!, time: false) : ""}";
  return VisualData(
    value: returnValue,
    widget: InkWell(
      onTap: () {
        Clipboard.setData(
          ClipboardData(
            text: returnValue,
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Date range copied to clipboard",
        );
      },
      child: Text(
        returnValue,
        style: const TextStyle(
          fontSize: 12,
        ),
      ),
    ),
  );
}

VisualData<String> visualPhoneNumber(
  dynamic value, {
  required BuildContext context,
}) {
  final phone = PhoneNumberModel.fromMap(
    DataClassUtils.dynamicToMapStringDynamic(value),
  );
  final returnValue =
      "${phone.flag}${phone.flag != null && phone.flag!.isNotEmpty ? " " : ""}${phone.prettyNumber ?? ''}";

  return VisualData(
    value: returnValue,
    widget: InkWell(
      onTap: () {
        Clipboard.setData(
          ClipboardData(
            text: phone.number,
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Phone number copied to clipboard",
        );
      },
      child: Text(
        returnValue,
        style: const TextStyle(
          fontSize: 12,
        ),
      ),
    ),
  );
}

VisualData<String> visualPhoneNumberList(
  dynamic value, {
  required BuildContext context,
}) {
  final returnValue = value is List
      ? value.map((e) {
          final phone = PhoneNumberModel.fromMap(
            DataClassUtils.dynamicToMapStringDynamic(e),
          );
          return "${phone.flag}${phone.flag != null && phone.flag!.isNotEmpty ? " " : ""}${phone.prettyNumber ?? ''}${(phone.flag != null && phone.flag!.isNotEmpty) || (phone.prettyNumber != null && phone.prettyNumber!.isNotEmpty) ? "\n" : ""}";
        }).join("")
      : "";

  final copyValue = value is List
      ? value.map((e) {
          final phone = PhoneNumberModel.fromMap(
            DataClassUtils.dynamicToMapStringDynamic(e),
          );
          return "${phone.number ?? ''}${(phone.number != null && phone.number!.isNotEmpty) ? "\n" : ""}";
        }).join("")
      : "";
  return VisualData(
    value: returnValue,
    widget: InkWell(
      onTap: () {
        Clipboard.setData(
          ClipboardData(
            text: copyValue,
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Phone numbers copied to clipboard",
        );
      },
      child: SelectableText(
        returnValue,
        style: const TextStyle(
          fontSize: 12,
        ),
      ),
    ),
  );
}
