// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import '../data_factory_index.dart';
import 'form_validators.dart';

class FormItem {
  final FormFF field;

  FormItem({required this.field});
}

extension SearchPath on FormFF {
  String get fullSearchPath {
    return "$name${searchPath != null && searchPath!.isNotEmpty ? ".$searchPath" : ""}";
  }
}

abstract class FormFF {
  final String name;
  final String displayName;
  final bool showOnTable;
  final bool showOnForm;
  final String? searchPath;
  final dynamic Function(Map<String, dynamic> rowData)? dataBuilder;
  final ValidatorFF? validator;
  final dynamic defaultValue;
  final FilterTypesFF filterType = FilterTypesFF.none;
  final double tableColumnWidth;
  final String? tip;

  const FormFF({
    required this.name,
    required this.displayName,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.dataBuilder,
    this.validator,
    this.defaultValue,
    this.tableColumnWidth = 100,
  });
}

enum SelectorColors {
  green,
  red,
  orange,
  yellow,
  grey,
}

class ColorSelectorFF implements FormFF {
  @override
  final String name;
  @override
  final String displayName;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;

  @override
  final String? defaultValue;
  @override
  final SelectorColors? Function(Map<String, dynamic> rowData)? dataBuilder;

  @override
  final ValidatorFF? validator;

  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  final List<SelectorColors> colors;

  final bool readOnly;

  ColorSelectorFF({
    required this.name,
    required this.displayName,
    required this.colors,
    this.searchPath,
    this.tip,
    this.defaultValue,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator,
    this.tableColumnWidth = 100,
    this.readOnly = false,
  });
}

class RichTextFF implements FormFF {
  @override
  final String name;
  @override
  final String displayName;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;

  @override
  final String? defaultValue;
  @override
  final String? Function(Map<String, dynamic> rowData)? dataBuilder;

  @override
  final StringValidatorFF validator;

  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  RichTextFF({
    required this.name,
    required this.displayName,
    this.searchPath,
    this.tip,
    this.defaultValue,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const StringValidatorFF(),
    this.tableColumnWidth = 160,
  });
}

class TextFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final String? Function(Map<String, dynamic> rowData)? dataBuilder;

  /// in order to format a string like this GHJ45GHJHN to GHJ-45-GHJHN use String like this: "###-00-#####"
  final String? textFormatter;

  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final StringValidatorFF validator;
  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  TextFF({
    required this.displayName,
    required this.name,
    this.searchPath,
    this.tip,
    this.defaultValue,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.textFormatter,
    this.validator = const StringValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class TextBoxFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final String? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final StringValidatorFF validator;

  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  final String? templateOptionsID;

  TextBoxFF({
    required this.displayName,
    required this.name,
    this.searchPath,
    this.tip,
    this.defaultValue,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const StringValidatorFF(),
    this.tableColumnWidth = 160,
    this.templateOptionsID,
  });
}

class FilesFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;
  @override
  final List<Map<String, dynamic>>? defaultValue;
  @override
  final List<Map<String, dynamic>>? Function(Map<String, dynamic> rowData)?
      dataBuilder;
  @override
  final NotNullValidatorFF? validator;

  @override
  final FilterTypesFF filterType = FilterTypesFF.none;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  final String path;

  // by default it will upload files via the verttice config global function
  final Future<FileModel> Function({
    required BuildContext context,
    required List<UploadFileDataModel> files,
  })? altUploadImageFun;

  final bool showDeleteFile;

  FilesFF({
    required this.displayName,
    required this.name,
    required this.path,
    this.altUploadImageFun,
    this.searchPath,
    this.tip,
    this.defaultValue,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator,
    this.tableColumnWidth = 160,
    this.showDeleteFile = false,
  });
}

class ImageFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;
  @override
  final Map<String, dynamic>? defaultValue;
  @override
  final Map<String, dynamic>? Function(Map<String, dynamic> rowData)?
      dataBuilder;
  @override
  final NotNullValidatorFF? validator;

  @override
  final FilterTypesFF filterType = FilterTypesFF.none;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  final String path;

  // by default it will upload files via the verttice config global function
  final Future<FileModel> Function({
    required BuildContext context,
    required UploadFileDataModel file,
  })? altUploadImageFun;

  ImageFF({
    required this.displayName,
    required this.name,
    required this.path,
    this.altUploadImageFun,
    this.searchPath,
    this.tip,
    this.defaultValue,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator,
    this.tableColumnWidth = 160,
  });
}

class SignPadFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;
  @override
  final Map<String, dynamic>? defaultValue;
  @override
  final Map<String, dynamic>? Function(Map<String, dynamic> rowData)?
      dataBuilder;
  @override
  final NotNullValidatorFF? validator;

  @override
  final FilterTypesFF filterType = FilterTypesFF.none;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  final String path;

  // by default it will upload files via the verttice config global function
  final Future<FileModel> Function({
    required BuildContext context,
    required UploadFileDataModel file,
  })? altUploadImageFun;

  SignPadFF({
    required this.displayName,
    required this.name,
    required this.path,
    this.altUploadImageFun,
    this.searchPath,
    this.tip,
    this.defaultValue,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator,
    this.tableColumnWidth = 160,
  });
}

class TextListFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final List<String>? defaultValue;
  @override
  final List<String>? Function(Map<String, dynamic> rowData)? dataBuilder;

  @override
  final BaseStringListValidatorFF validator;

  @override
  final FilterTypesFF filterType = FilterTypesFF.stringList;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  TextListFF({
    required this.displayName,
    required this.name,
    this.searchPath,
    this.tip,
    this.dataBuilder,
    this.showOnTable = true,
    this.defaultValue,
    this.showOnForm = true,
    this.validator = const StringListOrNullValidatorFF(),
    this.tableColumnWidth = 120,
  });
}

class SelectFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  final List<dynamic> options;
  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final String? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final StringValidatorFF validator;

  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  SelectFF({
    required this.displayName,
    required this.name,
    required this.options,
    this.defaultValue,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.tableColumnWidth = 100,
    this.validator = const StringValidatorFF(),
  });
}

class ReadonlyAddressOnMapFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final Map<String, dynamic>? Function(Map<String, dynamic> rowData)?
      dataBuilder;
  @override
  final BaseMapValidatorFF validator;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  @override
  final FilterTypesFF filterType = FilterTypesFF.string;
  ReadonlyAddressOnMapFF({
    required this.displayName,
    required this.name,
    this.defaultValue,
    this.dataBuilder,
    this.searchPath = 'fullAddress',
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const MapOrNullValidatorFF(mapValidator: {}),
    this.tableColumnWidth = 250,
  });
}

class AddressOnMapFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final Map<String, dynamic>? Function(Map<String, dynamic> rowData)?
      dataBuilder;
  @override
  final BaseMapValidatorFF validator;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  @override
  final FilterTypesFF filterType = FilterTypesFF.string;
  AddressOnMapFF({
    required this.displayName,
    required this.name,
    this.defaultValue,
    this.dataBuilder,
    this.searchPath = 'fullAddress',
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const MapOrNullValidatorFF(mapValidator: {}),
    this.tableColumnWidth = 250,
  });
}

class IntegerFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final String? searchPath;

  final String Function(double? value)? tableRenderAsString;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final int? defaultValue;
  @override
  final int? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final NumberValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.number;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  IntegerFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.tableRenderAsString,
    this.defaultValue = 0,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const NumberValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class DoubleFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final String? searchPath;

  final String Function(double? value)? tableRenderAsString;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final double? defaultValue;
  @override
  final double? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final NumberValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.number;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  DoubleFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.tableRenderAsString,
    this.defaultValue = 0.00,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const NumberValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class DateFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;

  final int? minDate;
  final int? maxDate;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;
  @override
  final int? defaultValue;
  @override
  final int? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final DateValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.date;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  DateFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.defaultValue,
    this.searchPath,
    this.tip,
    this.minDate,
    this.maxDate,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const DateValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class TimeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final int? defaultValue;
  @override
  final int? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final TimeValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.time;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  TimeFF({
    required this.displayName,
    required this.name,
    this.searchPath,
    this.tip,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.tableColumnWidth = 100,
    this.validator = const TimeValidatorFF(),
  });
}

class EmailFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final String? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final BaseEmailValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  EmailFF({
    required this.displayName,
    required this.name,
    this.searchPath,
    this.tip,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator = const EmailOrNullValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class PasswordFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final String? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final StringValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.none;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  PasswordFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator = const StringValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class PhoneNumberFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final Map<String, dynamic>? Function(Map<String, dynamic> rowData)?
      dataBuilder;
  @override
  final BasePhoneNumberValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  PhoneNumberFF({
    required this.displayName,
    required this.name,
    this.searchPath = 'prettyNumber',
    this.tip,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator = const PhoneNumberOrNullValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class ExpandSelectFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  final String optionsID;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final String? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final StringValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  final List<String>? defaultOptions;

  ExpandSelectFF({
    required this.displayName,
    required this.name,
    required this.optionsID,
    this.searchPath,
    this.tip,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator = const StringValidatorFF(),
    this.tableColumnWidth = 100,
    this.defaultOptions,
  });
}

class ExpandMultiSelectFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  final String optionsID;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;

  @override
  final List<String>? defaultValue;
  @override
  final String? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final BaseStringListValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;
  final List<String>? defaultOptions;

  ExpandMultiSelectFF({
    required this.displayName,
    required this.name,
    required this.optionsID,
    this.searchPath,
    this.tip,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator = const StringListOrNullValidatorFF(),
    this.tableColumnWidth = 150,
    this.defaultOptions,
  });
}

class RadioGroupFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  final List<dynamic> options;

  @override
  final String? searchPath;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? defaultValue;
  @override
  final String? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final StringValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  RadioGroupFF({
    required this.displayName,
    required this.name,
    required this.options,
    this.searchPath,
    this.tip,
    this.defaultValue,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const StringValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class CheckboxFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final bool? defaultValue;
  @override
  final bool? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.bool;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;
  CheckboxFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 100,
  });
}

class CustomFFData {
  final String displayName;
  final String name;
  final BuildContext context;
  final Map<String, dynamic> data;
  final void Function(String name, dynamic value) onChanged;
  final FormFieldState<dynamic>? state;
  final List<FormItem> columns;
  final String? tip;

  CustomFFData({
    required this.displayName,
    required this.name,
    required this.context,
    required this.data,
    required this.onChanged,
    required this.columns,
    this.tip,
    this.state,
  });
}

class CustomFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  final Widget Function(CustomFFData) builder;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final dynamic Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  final Future<void> Function({
    required BuildContext context,
    required void Function(List<FilterField> filter) filterCallback,
    required void Function() clearFilterCallback,
    List<FilterField>? filters,
  })? tableFilterDialog;

  final String Function({
    required BuildContext context,
    required List<FilterField> filters,
  })? tableFilterButtonLabelBuilder;

  CustomFF({
    required this.displayName,
    required this.name,
    required this.builder,
    this.tableFilterButtonLabelBuilder,
    this.tableFilterDialog,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.dataBuilder,
    this.defaultValue,
    this.showOnForm = true,
    this.validator,
    this.tableColumnWidth = 170,
    this.filterType = FilterTypesFF.string,
  });
}

class DateRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final dynamic defaultValue;
  @override
  final Map<String, int?>? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final MapOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.dateRange;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  DateRangeFF({
    required this.displayName,
    required this.name,
    this.searchPath,
    this.tip,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.tableColumnWidth = 150,
    this.validator = const MapOrNullValidatorFF(
      mapValidator: {
        "startDate": DateValidatorFF(),
        "endDate": DateValidatorFF()
      },
    ),
  });
}

class DateTimeRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final dynamic defaultValue;
  @override
  final Map<String, int?>? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final MapOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.dateRange;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  DateTimeRangeFF({
    required this.displayName,
    required this.name,
    this.searchPath,
    this.tip,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.tableColumnWidth = 150,
    this.validator = const MapOrNullValidatorFF(
      mapValidator: {
        "startDate": DateValidatorFF(),
        "endDate": DateValidatorFF()
      },
    ),
  });
}

class ReadonlyTimeRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final dynamic defaultValue;
  @override
  final Map<String, int?>? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final MapOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.timeRange;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  ReadonlyTimeRangeFF({
    required this.displayName,
    required this.name,
    this.searchPath,
    this.tip,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.tableColumnWidth = 100,
    this.validator = const MapOrNullValidatorFF(
      mapValidator: {
        "startTime": TimeValidatorFF(),
        "endTime": TimeValidatorFF()
      },
    ),
  });
}

class TimeRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final dynamic defaultValue;
  @override
  final Map<String, int?>? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final MapOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.timeRange;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  TimeRangeFF({
    required this.displayName,
    required this.name,
    this.searchPath,
    this.tip,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.tableColumnWidth = 100,
    this.validator = const MapOrNullValidatorFF(
      mapValidator: {
        "startTime": TimeValidatorFF(),
        "endTime": TimeValidatorFF()
      },
    ),
  });
}

class NumberRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final Map<String, double> defaultValue;
  @override
  final Map<String, num?>? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final MapOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.numberRange;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  NumberRangeFF({
    required this.displayName,
    required this.name,
    this.searchPath,
    this.tip,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue = const {"lowNumber": 0.00, "heighNumber": 0.00},
    this.tableColumnWidth = 130,
    this.validator = const MapOrNullValidatorFF(
      mapValidator: {
        "lowNumber": NumberValidatorFF(),
        "heighNumber": NumberValidatorFF()
      },
    ),
  });
}

class EmailListFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;
  @override
  final List<String>? defaultValue;
  @override
  final List<String>? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final StringListOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.stringList;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  EmailListFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.tableColumnWidth = 150,
    this.validator,
  });
}

class PhoneNumberListFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final List<String>? defaultValue;
  @override
  final List<Map<String, dynamic>?> Function(Map<String, dynamic> rowData)?
      dataBuilder;
  @override
  final StringListOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.stringList;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  PhoneNumberListFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath = 'prettyNumber',
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 150,
  });
}

class ReadonlyNumberRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final List<String>? defaultValue;
  @override
  final Map<String, num?>? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final MapOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.numberRange;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  ReadonlyNumberRangeFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.tableColumnWidth = 110,
    this.validator = const MapOrNullValidatorFF(
      mapValidator: {
        "lowNumber": NumberValidatorFF(),
        "heighNumber": NumberValidatorFF()
      },
    ),
  });
}

class ReadonlyPhoneNumberFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final List<String>? defaultValue;
  @override
  final Map<String, dynamic>? Function(Map<String, dynamic> rowData)?
      dataBuilder;
  @override
  final StringValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  ReadonlyPhoneNumberFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath = 'prettyNumber',
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 150,
  });
}

class ReadonlyDateFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;
  @override
  final int? defaultValue;
  @override
  final int? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.date;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  ReadonlyDateFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 100,
  });
}

class ReadonlyDateTimeRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;
  @override
  final int? defaultValue;
  @override
  final Map<String, int?>? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.dateRange;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  ReadonlyDateTimeRangeFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 150,
  });
}

class ReadonlyTimeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final int? defaultValue;
  @override
  final int? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.date;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  ReadonlyTimeFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 100,
  });
}

class ReadonlyNumberFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final int? defaultValue;
  @override
  final num? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.number;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;
  ReadonlyNumberFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 100,
  });
}

class ReadonlyPhoneNumberListFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;

  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final List<Map<String, dynamic>>? Function(Map<String, dynamic> rowData)?
      dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.stringList;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  ReadonlyPhoneNumberListFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 150,
  });
}

class ReadonlyTextListFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;

  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final List<String>? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.stringList;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  ReadonlyTextListFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 150,
  });
}

class ReadonlyTextFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;

  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final String? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  ReadonlyTextFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 150,
  });
}

class ReadonlyDateTimeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final int? defaultValue;
  @override
  final int? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.date;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  ReadonlyDateTimeFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 150,
  });
}

class ReadonlyDateRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final int? defaultValue;
  @override
  final Map<String, int?>? Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.date;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  ReadonlyDateRangeFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.searchPath,
    this.tip,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 150,
  });
}

class ParagraphFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.none;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  ParagraphFF({
    required this.displayName,
    this.name = "",
    this.searchPath,
    this.tip,
    this.showOnTable = false,
    this.showOnForm = true,
    this.defaultValue,
    this.dataBuilder,
    this.validator,
    this.tableColumnWidth = 100,
  });
}

class ParagraphHeaderFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final String? searchPath;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.none;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;

  ParagraphHeaderFF({
    required this.displayName,
    this.name = "",
    this.searchPath,
    this.tip,
    this.showOnTable = false,
    this.showOnForm = true,
    this.defaultValue,
    this.dataBuilder,
    this.validator,
    this.tableColumnWidth = 100,
  });
}

class NoneFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? searchPath;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final dynamic defaultValue;
  @override
  final FilterTypesFF filterType = FilterTypesFF.none;

  @override
  final double tableColumnWidth;
  @override
  final String? tip;
  NoneFF({
    this.displayName = "",
    this.name = "",
    this.searchPath,
    this.tip,
    this.showOnTable = false,
    this.showOnForm = true,
    this.dataBuilder,
    this.validator,
    this.defaultValue,
    this.tableColumnWidth = 100,
  });
}
