// 📦 Package imports:
import "package:validators/validators.dart";
import 'package:flutter_multi_formatter/flutter_multi_formatter.dart';
import 'package:verttice_logic/verttice_logic.dart';

abstract class ValidatorFF {
  String? valid(dynamic data);
}

class NotNullValidatorFF implements ValidatorFF {
  const NotNullValidatorFF();
  @override
  String? valid(dynamic data) {
    if (data == null) {
      return "Value can't be empty";
    }
    return null;
  }
}

abstract class BasePhoneNumberValidatorFF implements ValidatorFF {
  @override
  String? valid(dynamic data);
}

class PhoneNumberOrNullValidatorFF implements BasePhoneNumberValidatorFF {
  const PhoneNumberOrNullValidatorFF();
  @override
  String? valid(dynamic data) {
    if (data != null &&
        !isPhoneValid(
          DataClassUtils.dynamicToString(data),
        )) {
      return "Enter a valid phone number";
    }
    return null;
  }
}

class PhoneNumberValidatorFF implements BasePhoneNumberValidatorFF {
  const PhoneNumberValidatorFF();
  @override
  String? valid(dynamic data) {
    if (!isPhoneValid(
      DataClassUtils.dynamicToString(data),
    )) {
      return "Enter a valid phone number";
    }
    return null;
  }
}

abstract class BaseEmailValidatorFF implements ValidatorFF {
  @override
  String? valid(dynamic data);
}

class EmailOrNullValidatorFF implements BaseEmailValidatorFF {
  const EmailOrNullValidatorFF();
  @override
  String? valid(dynamic data) {
    final dataAsString = DataClassUtils.dynamicToString(data);
    return dataAsString == "" || isEmail(dataAsString)
        ? null
        : "Please enter a valid email";
  }
}

class EmailValidatorFF implements BaseEmailValidatorFF {
  const EmailValidatorFF();
  @override
  String? valid(dynamic data) {
    final dataAsString = DataClassUtils.dynamicToString(data);
    return isEmail(dataAsString) ? null : "Please enter a valid email";
  }
}

class StringValidatorFF implements ValidatorFF {
  final int minLength;
  final int? maxLength;

  const StringValidatorFF({this.minLength = 0, this.maxLength});

  @override
  String? valid(dynamic data) {
    final dataAsString = DataClassUtils.dynamicToString(data);
    if (dataAsString.length < minLength) {
      return "Value must be longer then ${minLength.toString()} characters";
    } else if (maxLength != null && dataAsString.length > maxLength!) {
      return "Value must to be shorter then ${maxLength.toString()} characters";
    }
    return null;
  }
}

class NumberValidatorFF implements ValidatorFF {
  final int min;
  final int? max;

  const NumberValidatorFF({this.min = 0, this.max});

  @override
  String? valid(dynamic data) {
    final dataAsNumber = DataClassUtils.dynamicToDouble(data, defaultValue: 0)!;
    if (dataAsNumber < min) {
      return "Value must to be bigger then ${min.toString()}";
    } else if (max != null && dataAsNumber > max!) {
      return "Value must to be less then ${max.toString()}";
    }
    return null;
  }
}

class DateValidatorFF implements ValidatorFF {
  final int? minDateMilisconds;
  final int? maxDateMilisconds;

  const DateValidatorFF({this.minDateMilisconds, this.maxDateMilisconds});

  @override
  String? valid(dynamic data) {
    final dataAsNumber = DataClassUtils.dynamicToDouble(data, defaultValue: 0)!;
    if (minDateMilisconds != null && dataAsNumber < minDateMilisconds!) {
      return "Value must to be bigger then ${Dates.formatDataAndTime(milliseconds: minDateMilisconds ?? 0, time: false)}";
    } else if (maxDateMilisconds != null && dataAsNumber > maxDateMilisconds!) {
      return "Value must to be less then ${Dates.formatDataAndTime(milliseconds: maxDateMilisconds ?? 0, time: false)}";
    }
    return null;
  }
}

class TimeValidatorFF implements ValidatorFF {
  final int? minTimeMilisconds;
  final int? maxTimeMilisconds;

  const TimeValidatorFF({this.minTimeMilisconds, this.maxTimeMilisconds});

  @override
  String? valid(dynamic data) {
    final dataAsNumber = DataClassUtils.dynamicToDouble(data, defaultValue: 0)!;
    if (minTimeMilisconds != null && dataAsNumber < minTimeMilisconds!) {
      return "Value must to be bigger then ${Dates.formatDataAndTime(milliseconds: minTimeMilisconds ?? 0, date: false)}";
    } else if (maxTimeMilisconds != null && dataAsNumber > maxTimeMilisconds!) {
      return "Value must to be less then ${Dates.formatDataAndTime(milliseconds: maxTimeMilisconds ?? 0, date: false)}";
    }
    return null;
  }
}

abstract class BaseStringListValidatorFF implements ValidatorFF {
  @override
  String? valid(dynamic data);
}

class StringListOrNullValidatorFF implements BaseStringListValidatorFF {
  final int minLength;
  final int? maxLength;
  const StringListOrNullValidatorFF({
    this.minLength = 0,
    this.maxLength,
  });

  @override
  String? valid(dynamic data) {
    final dataAsString = DataClassUtils.dynamicToStringList(data);
    if (dataAsString.length < minLength) {
      return "Value must to be longer then ${minLength.toString()} items";
    } else if (maxLength != null && dataAsString.length > maxLength!) {
      return "Value must to be shorter then ${maxLength.toString()} items";
    }
    return null;
  }
}

class StringListValidatorFF implements BaseStringListValidatorFF {
  final int minLength;
  final int? maxLength;

  StringListValidatorFF({this.minLength = 0, this.maxLength});

  @override
  String? valid(dynamic data) {
    final dataAsString = DataClassUtils.dynamicToString(data);
    if (dataAsString.length < minLength) {
      return "Value must to be longer then ${minLength.toString()} items";
    } else if (maxLength != null && dataAsString.length > maxLength!) {
      return "Value must to be shorter then ${maxLength.toString()} items";
    }
    return null;
  }
}

abstract class BaseMapValidatorFF extends ValidatorFF {
  @override
  String? valid(dynamic data);
}

class MapOrNullValidatorFF implements BaseMapValidatorFF {
  final Map<String, ValidatorFF> mapValidator;

  const MapOrNullValidatorFF({
    required this.mapValidator,
  });

  @override
  String? valid(dynamic data) {
    final dataAsMap = DataClassUtils.dynamicToMapStringDynamic(data);
    List<String> error = [];
    for (var element in mapValidator.entries) {
      if (dataAsMap[element.key] != null &&
          element.value.valid(dataAsMap[element.key]) != null) {
        String currentError = element.value.valid(dataAsMap[element.key])!;
        error.add("Field ${element.key}, $currentError");
      }
    }

    return error.isNotEmpty ? error.join(", ") : null;
  }
}

class MaptValidatorFF implements BaseMapValidatorFF {
  final Map<String, ValidatorFF> mapValidator;

  const MaptValidatorFF({
    required this.mapValidator,
  });

  @override
  String? valid(dynamic data) {
    final dataAsMap = DataClassUtils.dynamicToMapStringDynamic(data);
    List<String> error = [];
    for (var element in mapValidator.entries) {
      if (dataAsMap[element.key] == null ||
          element.value.valid(dataAsMap[element.key]) != null) {
        if (dataAsMap[element.key] == null) {
          error.add("Field ${element.key} can\"t be empty");
        } else {
          String currentError = element.value.valid(dataAsMap[element.key])!;
          error.add("Field ${element.key}, $currentError");
        }
      }
    }

    return error.isNotEmpty ? error.join(", ") : null;
  }
}
