// 🐦 Flutter imports:
import "package:flutter/material.dart";
import 'package:flutter/services.dart';

// 📦 Package imports:
import "package:collection/collection.dart";
import 'package:day_night_time_picker/day_night_time_picker.dart';
import 'package:day_night_time_picker/lib/constants.dart';
import 'package:verttice_logic/verttice_logic.dart';

// 🌎 Project imports:
import "package:verttice_ui/src/const/icons.dart";
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

class DateTimeRangeFormFactory extends StatefulWidget {
  final Function(String name, dynamic value) onChanged;
  final String displayName;
  final String name;
  final dynamic value;
  final FormFieldState<dynamic>? state;
  final String? tip;

  const DateTimeRangeFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.value,
    this.state,
    this.tip,
  }) : super(key: key);

  @override
  State<DateTimeRangeFormFactory> createState() =>
      _DateTimeRangeFormFactoryState();
}

class _DateTimeRangeFormFactoryState extends State<DateTimeRangeFormFactory> {
  var _value = <String, int?>{};

  @override
  void initState() {
    super.initState();
    _value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value)
        .cast<String, int?>();
  }

  @override
  void didUpdateWidget(covariant DateTimeRangeFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    final value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value);
    if (!const MapEquality().equals(_value, value)) {
      _value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value)
          .cast<String, int?>();
    }
  }

  @override
  Widget build(BuildContext context) {
    const textStyle = TextStyle(fontSize: 12);
    final iconStyle = IconStyle(size: 12);
    return BorderLabel(
      label: widget.displayName,
      tip: widget.tip,
      onCopied: () async {
        await Clipboard.setData(
          ClipboardData(
            text:
                "${_value["startDate"] != null && _value["startDate"] is int ? Dates.formatDataAndTime(milliseconds: _value["startDate"]!) : ""} - ${_value["endDate"] != null && _value["endDate"] is int ? Dates.formatDataAndTime(milliseconds: _value["endDate"]!) : ""}",
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Copied to clipboard",
        );
      },
      onClear: () {
        widget.onChanged(widget.name, null);
        widget.state?.didChange(null);
      },
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          Row(
            children: [
              ThemedCleanTextButton(
                _value["startDate"] != null && _value["startDate"] is int
                    ? Dates.formatDataAndTime(
                        milliseconds: _value["startDate"]!,
                        time: false,
                      )
                    : "Start Date",
                textStyle: textStyle,
                icon: ThemedIcons.calendar(
                  iconStyle: iconStyle,
                ),
                onPressed: () async {
                  DateTime? initialDate;
                  if (_value['startDate'] != null &&
                      _value['startDate'] is int) {
                    initialDate = DateTime.fromMillisecondsSinceEpoch(
                      _value['startDate']!,
                    );
                  }
                  DateTime? maxDate;
                  if (_value['endDate'] != null && _value['endDate'] is int) {
                    maxDate =
                        DateTime.fromMillisecondsSinceEpoch(_value['endDate']!);
                  }

                  DateTime? date = await showDatePicker(
                    context: context,
                    initialDate: initialDate ?? DateTime.now(),
                    firstDate: DateTime(1950),
                    lastDate: maxDate ?? DateTime(2100),
                  );

                  if (date == null) {
                    return;
                  }

                  final choosenDate = {
                    ..._value,
                    "startDate": date.toUtc().millisecondsSinceEpoch,
                    if (maxDate == null ||
                        (maxDate.day == date.day &&
                            maxDate.month == date.month &&
                            maxDate.year == date.year))
                      "endDate": DateTime(
                        date.year,
                        date.month,
                        date.day,
                        date.hour + 3,
                        date.minute,
                      ).toUtc().millisecondsSinceEpoch,
                  };
                  widget.onChanged(widget.name, choosenDate);
                  widget.state?.didChange(choosenDate);
                },
              ),
              if (_value["startDate"] != null && _value["startDate"] is int)
                ThemedCleanTextButton(
                  _value["startDate"] != null && _value["startDate"] is int
                      ? Dates.formatDataAndTime(
                          milliseconds: _value["startDate"]!,
                          date: false,
                        )
                      : "Start Time",
                  textStyle: textStyle,
                  icon: ThemedIcons.clock(
                    iconStyle: iconStyle,
                  ),
                  onPressed: () async {
                    DateTime? maxDate;
                    if (_value['endDate'] != null && _value['endDate'] is int) {
                      maxDate = DateTime.fromMillisecondsSinceEpoch(
                          _value['endDate']!);
                    }
                    await Navigator.of(context).push(
                      showPicker(
                        context: context,
                        is24HrFormat: true,
                        minuteInterval: MinuteInterval.FIVE,
                        value: TimeOfDay.fromDateTime(
                          DateTime.fromMillisecondsSinceEpoch(
                            _value['startDate']!,
                          ),
                        ),
                        onChange: (time) {
                          final currentDateLocal =
                              DateTime.fromMillisecondsSinceEpoch(
                            _value['startDate']!,
                          ).toLocal();

                          DateTime date = DateTime(
                            currentDateLocal.year,
                            currentDateLocal.month,
                            currentDateLocal.day,
                            time.hour,
                            time.minute,
                          );

                          final choosenDate = {
                            ..._value,
                            "startDate": date.toUtc().millisecondsSinceEpoch,
                            if (maxDate == null ||
                                (maxDate.day == date.day &&
                                    maxDate.month == date.month &&
                                    maxDate.year == date.year))
                              "endDate": DateTime(
                                date.year,
                                date.month,
                                date.day,
                                date.hour + 3,
                                date.minute,
                              ).toUtc().millisecondsSinceEpoch,
                          };
                          widget.onChanged(widget.name, choosenDate);
                          widget.state?.didChange(choosenDate);
                        },
                      ),
                    );
                  },
                ),
            ],
          ),
          if (_value['startDate'] != null &&
              _value['startDate'] is int &&
              _value['endDate'] != null &&
              _value['endDate'] is int)
            Row(
              children: [
                ThemedCleanTextButton(
                  Dates.formatDataAndTime(
                    milliseconds: _value["endDate"]!,
                    time: false,
                  ),
                  textStyle: textStyle,
                  icon: ThemedIcons.calendar(
                    iconStyle: iconStyle,
                  ),
                  onPressed: () async {
                    DateTime? initialDate;
                    if (_value['endDate'] != null && _value['endDate'] is int) {
                      initialDate = DateTime.fromMillisecondsSinceEpoch(
                        _value['endDate']!,
                      );
                    }
                    final minDate = DateTime.fromMillisecondsSinceEpoch(
                      _value['startDate']!,
                    );

                    DateTime? date = await showDatePicker(
                      context: context,
                      initialDate: initialDate ?? DateTime.now(),
                      firstDate: minDate,
                      lastDate: DateTime(2100),
                    );

                    if (date == null) {
                      return;
                    }

                    final choosenDate = {
                      ..._value,
                      "endDate": DateTime.utc(
                        date.year,
                        date.month,
                        date.day,
                        date.isBefore(minDate) || date.isAtSameMomentAs(minDate)
                            ? minDate.hour + 3
                            : date.hour,
                        date.minute,
                      ).toUtc().millisecondsSinceEpoch,
                    };

                    widget.onChanged(widget.name, choosenDate);
                    widget.state?.didChange(choosenDate);
                  },
                ),
                ThemedCleanTextButton(
                  Dates.formatDataAndTime(
                    milliseconds: _value["endDate"]!,
                    date: false,
                  ),
                  textStyle: textStyle,
                  icon: ThemedIcons.clock(
                    iconStyle: iconStyle,
                  ),
                  onPressed: () async {
                    DateTime? minDate;
                    if (_value['startDate'] != null &&
                        _value['startDate'] is int) {
                      DateTime? maxDate;
                      if (_value['endDate'] != null &&
                          _value['endDate'] is int) {
                        maxDate = DateTime.fromMillisecondsSinceEpoch(
                          _value['endDate']!,
                        );
                      }
                      minDate = DateTime.fromMillisecondsSinceEpoch(
                        _value['startDate']!,
                      );
                      if (maxDate != null &&
                          ((maxDate.day != minDate.day) ||
                              (maxDate.day == minDate.day &&
                                  (maxDate.month != minDate.month ||
                                      maxDate.year != minDate.year)))) {
                        minDate = null;
                      }
                    }

                    await Navigator.of(context).push(
                      showPicker(
                        context: context,
                        is24HrFormat: true,
                        minHour: minDate?.hour.toDouble() ?? double.infinity,
                        minuteInterval: MinuteInterval.FIVE,
                        value: TimeOfDay.fromDateTime(
                          DateTime.fromMillisecondsSinceEpoch(
                            _value['endDate']!,
                          ),
                        ),
                        onChange: (time) {
                          final currentDateLocal =
                              DateTime.fromMillisecondsSinceEpoch(
                            _value['endDate']!,
                          ).toLocal();
                          DateTime date = DateTime(
                            currentDateLocal.year,
                            currentDateLocal.month,
                            currentDateLocal.day,
                            time.hour,
                            time.minute,
                          );

                          final choosenDate = {
                            ..._value,
                            "endDate": date.toUtc().millisecondsSinceEpoch,
                          };
                          widget.onChanged(widget.name, choosenDate);
                          widget.state?.didChange(choosenDate);
                        },
                      ),
                    );
                  },
                ),
              ],
            ),
        ],
      ),
    );
  }
}
