// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:verttice_logic/verttice_logic.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

import 'expand_select_add_option.dart';

class ExpandSelectFormFactory extends StatefulWidget {
  final Function(String name, dynamic value) onChanged;
  final String displayName;
  final String name;
  final String optionsID;
  final String value;
  final FormFieldState<dynamic>? state;
  final List<String>? defaultOptions;
  final String? tip;

  const ExpandSelectFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.optionsID,
    this.value = "",
    this.state,
    this.defaultOptions,
    this.tip,
  }) : super(key: key);

  @override
  State<ExpandSelectFormFactory> createState() => _ExpandSelectState();
}

class _ExpandSelectState extends State<ExpandSelectFormFactory> {
  String _value = "";
  final String _loadKey = IDs.l16();

  @override
  void initState() {
    super.initState();
    _value = DataClassUtils.dynamicToString(widget.value);
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant ExpandSelectFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = DataClassUtils.dynamicToString(widget.value);
    }
  }

  Future<void> _addOption({required List<String> currentOptions}) async {
    await sizedDialog(
      context: context,
      height: 400,
      width: 430,
      builder: (BuildContext context) {
        return ExpandSelectAddOption(
          displayName: widget.displayName,
          optionsID: widget.optionsID,
          currentOptions: currentOptions,
          onDone: () {
            Requery.queryCache.invalidateQueries(_loadKey);
          },
        );
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      tip: widget.tip,
      child: SelectableText(_value),
      onPressed: () async {
        await sizedDialog(
          context: context,
          height: 400,
          width: 430,
          builder: (BuildContext context) {
            return Requery.query<List<String>>(
              key: _loadKey,
              future: () =>
                  VertticeUIConfig.instance.expandableOptionsConfig.getOptions(
                optionsID: widget.optionsID,
              ),
              builder: ({
                required BuildContext context,
                required RequeryResponse<List<String>> response,
              }) {
                if (response.loading) {
                  return const ThemedMessage(
                    showLoading: true,
                  );
                }

                if (response.error != null) {
                  return ThemedMessage(
                    message: "Failed to get options",
                    actions: [
                      ThemedIconButton(
                        onPressed: () {
                          Requery.queryCache.invalidateQueries(_loadKey);
                        },
                        icon: ThemedIcons.refresh(),
                      )
                    ],
                  );
                }
                final options = [...?widget.defaultOptions, ...?response.data];

                if (options.isEmpty) {
                  return ThemedMessage(
                    message: "No options yet",
                    actions: [
                      ThemedIconTextButton(
                        ThemedIcons.add(),
                        'Add Option',
                        onPressed: () async {
                          await _addOption(
                            currentOptions: [],
                          );
                        },
                      ),
                      sizedBoxW2,
                      ThemedIconTextButton(
                        ThemedIcons.refresh(),
                        'Refresh List',
                        onPressed: () {
                          Requery.queryCache.invalidateQueries(_loadKey);
                        },
                      ),
                    ],
                  );
                }
                return Scaffold(
                  appBar: ThemedAppBar(
                    navigateBackButton: true,
                    title: "Select ${widget.displayName}",
                    actions: [
                      ThemedIconButton(
                        icon: ThemedIcons.delete(),
                        onPressed: () async {
                          final optionsToDelete = await VertticeUIConfig
                              .instance.expandableOptionsConfig
                              .getOptions(
                            optionsID: widget.optionsID,
                          );
                          String? optionToDelete;
                          await sizedDialog<String?>(
                            context: context,
                            height: 400,
                            width: 430,
                            builder: (BuildContext context) {
                              return SelectFormFactory<String>(
                                displayName: "Select Option To Delete",
                                name: "selectOptionToDelete",
                                onChanged: (String name, String? value) {
                                  optionToDelete = value;
                                  Navigator.of(context).pop();
                                },
                                options: Set<String>.from(
                                  optionsToDelete.map((e) => e).toList(),
                                ).toList(),
                              );
                            },
                          );
                          if (optionToDelete != null) {
                            LoadingDialog.instance.openDialog(
                              context: context,
                              loadingMessage:
                                  "Deleting option '$optionToDelete'",
                            );
                            try {
                              await VertticeUIConfig
                                  .instance.expandableOptionsConfig
                                  .deleteOption(
                                optionsID: widget.optionsID,
                                option: optionToDelete!,
                              );

                              ThemedToast.success(
                                context: context,
                                message: "Deleted option '$optionToDelete'",
                              );
                              Requery.queryCache.invalidateQueries(_loadKey);
                            } catch (e) {
                              ThemedToast.error(
                                context: context,
                                message:
                                    "Failed to delete option '$optionToDelete'",
                              );
                            } finally {
                              LoadingDialog.instance.closeDialog();
                            }
                          }
                        },
                      ),
                      ThemedIconButton(
                        icon: ThemedIcons.add(),
                        onPressed: () async {
                          await _addOption(currentOptions: options);
                        },
                      ),
                    ],
                  ),
                  body: Center(
                    child: Padding(
                      padding: const EdgeInsets.all(6),
                      child: SizedBox(
                        height: 130,
                        width: 430,
                        child: SelectFormFactory(
                          displayName: "Select Option",
                          name: "selectOption",
                          value: _value,
                          onChanged: (String name, dynamic value) {
                            widget.onChanged(widget.name, value);
                            widget.state?.didChange(value);
                            Navigator.of(context).pop();
                          },
                          options: Set<String>.from(
                            options.map((e) => e.toLowerCase()).toList(),
                          ).toList().map((e) => e.titleCase).toList(),
                        ),
                      ),
                    ),
                  ),
                );
              },
            );
          },
        );
      },
    );
  }
}
