// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import "package:collection/collection.dart";
import 'package:after_layout/after_layout.dart';
import 'package:provider/provider.dart';
import 'package:verttice_logic/verttice_logic.dart';

// 🌎 Project imports:
import 'package:verttice_ui/src/const/box_sizes.dart';
import 'package:verttice_ui/src/state/verttice_ui_state.dart';
import 'package:verttice_ui/src/widgets/data_factory/common/form_item_to_filter_item.dart';
import '../data_factory_index.dart';
export '../factory_items/factory_items_index.dart';

class FormFactory extends StatefulWidget {
  final Map<String, dynamic> formState;
  final List<FormItem> formItems;
  final bool gridView;
  final int gridViewItemWidth;
  final bool isFilter;
  final bool returnFormStateOnLoad;
  final Function(String name, dynamic value) onChanged;
  final bool readOnly;

  const FormFactory({
    Key? key,
    required this.formState,
    required this.formItems,
    required this.onChanged,
    this.gridViewItemWidth = 350,
    this.gridView = false,
    this.isFilter = false,
    this.returnFormStateOnLoad = false,
    this.readOnly = false,
  }) : super(key: key);

  @override
  FormFactoryState createState() => FormFactoryState();
}

class FormFactoryState extends State<FormFactory>
    with AfterLayoutMixin<FormFactory> {
  Map<String, dynamic> _formState = {};
  Map<String, FormFieldValidator> _formFieldState = {};

  @override
  void initState() {
    super.initState();
    _setFormStateData();
  }

  @override
  void afterFirstLayout(BuildContext context) {
    if (widget.returnFormStateOnLoad) {
      for (var currentFormItem in widget.formItems) {
        if (currentFormItem.field.name.isNotEmpty) {
          onChanged(
            currentFormItem.field.name,
            _formState[currentFormItem.field.name],
          );
        }
      }
    }
  }

  _setFormStateData() {
    _formState = {};
    for (var currentFormItem in widget.formItems) {
      if (currentFormItem.field.name.isNotEmpty) {
        _formState[currentFormItem.field.name] =
            widget.formState[currentFormItem.field.name] ??
                currentFormItem.field.defaultValue;
      }
    }
  }

  @override
  void didUpdateWidget(covariant FormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    final eq = const MapEquality().equals;
    if (!eq(_formState, widget.formState)) {
      _setFormStateData();
    }
  }

  onChanged(String name, dynamic value) {
    widget.onChanged(name, value);
    _formState[name] = value;
  }

  bool isValid() {
    bool valid = true;
    for (var element in _formFieldState.entries) {
      if (element.value(_formState[element.key]) != null) {
        valid = false;
        break;
      }
    }
    return valid;
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (context, constraints) {
      _formFieldState = {};
      final fields = <Widget>[];
      for (var element in widget.formItems) {
        if (element.field.showOnForm) {
          final validator = element.field.validator;
          final name = element.field.name;
          _formFieldState[name] = (dynamic data) {
            return validator?.valid(data);
          };
          fields.add(
            FormField(
              autovalidateMode: AutovalidateMode.onUserInteraction,
              validator: _formFieldState[name],
              builder: (FormFieldState<dynamic> state) {
                return SizedBox(
                  width: widget.gridView
                      ? widget.gridViewItemWidth.toDouble()
                      : constraints.maxWidth - 14,
                  height: 130,
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Expanded(
                        child: _FormItem(
                          columns: widget.formItems,
                          formItem: element,
                          onChanged: onChanged,
                          formState: _formState,
                          isFilter: widget.isFilter,
                          readOnly: widget.readOnly,
                          state: state,
                        ),
                      ),
                      Padding(
                        padding: const EdgeInsets.symmetric(
                          vertical: 1.5,
                          horizontal: 4,
                        ),
                        child: SelectableText(
                          _formFieldState[name] != null
                              ? _formFieldState[name]!(_formState[name]) ?? ""
                              : "",
                          maxLines: 1,
                          style: const TextStyle(
                            color: Colors.red,
                            fontSize: 12,
                            overflow: TextOverflow.fade,
                          ),
                        ),
                      )
                    ],
                  ),
                );
              },
            ),
          );
        }
      }

      return SizedBox(
        width: constraints.maxWidth,
        child: Consumer<VertticeUIState>(builder: (context, value, _) {
          return Padding(
            padding: const EdgeInsets.only(left: 7, right: 7),
            child: Wrap(
              runAlignment: WrapAlignment.spaceAround,
              alignment: WrapAlignment.spaceAround,
              runSpacing: 8,
              spacing: 5,
              direction: widget.gridView ? Axis.horizontal : Axis.vertical,
              children: fields,
            ),
          );
        }),
      );
    });
  }
}

class _FormItem extends StatelessWidget {
  final FormItem formItem;
  final bool isFilter;
  final bool readOnly;
  final Map<String, dynamic> formState;
  final Function(String name, dynamic value) onChanged;
  final FormFieldState<dynamic> state;
  final List<FormItem> columns;
  const _FormItem({
    Key? key,
    required this.formItem,
    required this.onChanged,
    required this.formState,
    required this.state,
    required this.columns,
    required this.readOnly,
    required this.isFilter,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    FormItem currentFormItem = formItem;
    if (readOnly) {
      currentFormItem = formItemToReadonlyItem(formItem: formItem);
    } else if (isFilter) {
      currentFormItem = formItemToFilterItem(formItem: formItem);
    }

    dynamic value;
    if (currentFormItem.field.dataBuilder != null) {
      value = currentFormItem.field.dataBuilder!(formState);
    } else {
      value = formState[currentFormItem.field.name];
    }
    if (currentFormItem.field is FilesFF) {
      return FilesFormFactory(
        value: value,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        path: (currentFormItem.field as FilesFF).path,
        altUploadImageFun: (currentFormItem.field as FilesFF).altUploadImageFun,
        state: state,
        onChanged: onChanged,
        tip: currentFormItem.field.tip,
        showDeleteFile: (currentFormItem.field as FilesFF).showDeleteFile,
      );
    } else if (currentFormItem.field is ImageFF) {
      return ImageFormFactory(
        value: value,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        path: (currentFormItem.field as ImageFF).path,
        altUploadImageFun: (currentFormItem.field as ImageFF).altUploadImageFun,
        state: state,
        onChanged: onChanged,
        tip: currentFormItem.field.tip,
      );
    } else if (currentFormItem.field is SignPadFF) {
      return SignPadFormFactory(
        value: value,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        path: (currentFormItem.field as SignPadFF).path,
        altUploadImageFun:
            (currentFormItem.field as SignPadFF).altUploadImageFun,
        state: state,
        onChanged: onChanged,
        tip: currentFormItem.field.tip,
      );
    } else if (currentFormItem.field is TextFF) {
      return TextFormFactory(
        value: DataClassUtils.dynamicToString(value, defaultValue: ""),
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        type: TextFieldInputFF.text,
        textFormatter: (currentFormItem.field as TextFF).textFormatter,
        state: state,
        onChanged: onChanged,
        tip: currentFormItem.field.tip,
      );
    } else if (currentFormItem.field is EmailFF) {
      return TextFormFactory(
        value: DataClassUtils.dynamicToString(value, defaultValue: ""),
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        type: TextFieldInputFF.email,
        onChanged: onChanged,
        state: state,
        tip: currentFormItem.field.tip,
      );
    } else if (currentFormItem.field is IntegerFF) {
      return TextFormFactory(
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        type: TextFieldInputFF.integer,
        onChanged: onChanged,
        state: state,
        value: DataClassUtils.dynamicToString(value, defaultValue: ""),
        tip: currentFormItem.field.tip,
      );
    } else if (currentFormItem.field is DoubleFF) {
      return TextFormFactory(
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        type: TextFieldInputFF.double,
        onChanged: onChanged,
        state: state,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToString(value, defaultValue: ""),
      );
    } else if (currentFormItem.field is TextBoxFF) {
      return TextBoxFormFactory(
        templateOptionsID:
            (currentFormItem.field as TextBoxFF).templateOptionsID,
        value: DataClassUtils.dynamicToString(value, defaultValue: ""),
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        state: state,
        tip: currentFormItem.field.tip,
        onChanged: onChanged,
      );
    } else if (currentFormItem.field is SelectFF) {
      return SelectFormFactory(
        onChanged: onChanged,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        options: (currentFormItem.field as SelectFF).options,
        state: state,
        tip: currentFormItem.field.tip,
        value: value,
      );
    } else if (currentFormItem.field is RadioGroupFF) {
      return RadioGroupFormFactory(
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        options: DataClassUtils.dynamicToStringList(
            (currentFormItem.field as RadioGroupFF).options),
        onChanged: onChanged,
        state: state,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToString(value),
      );
    } else if (currentFormItem.field is ExpandSelectFF) {
      return ExpandSelectFormFactory(
        onChanged: onChanged,
        defaultOptions:
            (currentFormItem.field as ExpandSelectFF).defaultOptions,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        state: state,
        tip: currentFormItem.field.tip,
        optionsID: (currentFormItem.field as ExpandSelectFF).optionsID,
        value: DataClassUtils.dynamicToString(value),
      );
    } else if (currentFormItem.field is ExpandMultiSelectFF) {
      return ExpandMultiSelectFormFactory(
        onChanged: onChanged,
        displayName: currentFormItem.field.displayName,
        defaultOptions:
            (currentFormItem.field as ExpandMultiSelectFF).defaultOptions,
        name: currentFormItem.field.name,
        state: state,
        tip: currentFormItem.field.tip,
        optionsID: (currentFormItem.field as ExpandMultiSelectFF).optionsID,
        value: DataClassUtils.dynamicToStringList(value),
      );
    } else if (currentFormItem.field is CheckboxFF) {
      return CheckboxFormFactory(
        onChanged: onChanged,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        value: value,
        tip: currentFormItem.field.tip,
        state: state,
      );
    } else if (currentFormItem.field is CustomFF) {
      return (currentFormItem.field as CustomFF).builder(CustomFFData(
        context: context,
        columns: columns,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        onChanged: onChanged,
        state: state,
        tip: currentFormItem.field.tip,
        data: formState,
      ));
    } else if (currentFormItem.field is DateRangeFF) {
      return DateRangeFormFactory(
        onChanged: onChanged,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        state: state,
        tip: currentFormItem.field.tip,
        value: value,
      );
    } else if (currentFormItem.field is DateTimeRangeFF) {
      return DateTimeRangeFormFactory(
        onChanged: onChanged,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        value: value,
        tip: currentFormItem.field.tip,
        state: state,
      );
    } else if (currentFormItem.field is TimeRangeFF) {
      return TimeRangeFormFactory(
        onChanged: onChanged,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        state: state,
        tip: currentFormItem.field.tip,
        value: value,
      );
    } else if (currentFormItem.field is NumberRangeFF) {
      return NumberRangeFormFactory(
        onChanged: onChanged,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        state: state,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToMapStringDoubleOrNull(
          value,
          defaultValue: {
            "lowNumber": null,
            "heighNumber": null,
          },
        ),
      );
    } else if (currentFormItem.field is AddressOnMapFF) {
      return AddressOnMapFormFactory(
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        state: state,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToMapStringDynamic(value),
        onChanged: onChanged,
      );
    } else if (currentFormItem.field is ReadonlyDateRangeFF) {
      return ReadonlyFormFactory(
        type: ReadonlyFormType.dateRange,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToMapStringIntOrNull(
          value,
          defaultValue: null,
        ),
      );
    } else if (currentFormItem.field is ReadonlyTimeRangeFF) {
      return ReadonlyFormFactory(
        type: ReadonlyFormType.timeRange,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToMapStringIntOrNull(
          value,
          defaultValue: null,
        ),
      );
    } else if (currentFormItem.field is ReadonlyDateTimeRangeFF) {
      return ReadonlyFormFactory(
        type: ReadonlyFormType.dateTimeRange,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        tip: currentFormItem.field.tip,
        value:
            DataClassUtils.dynamicToMapStringDynamic(value, defaultValue: null),
      );
    } else if (currentFormItem.field is ReadonlyNumberRangeFF) {
      return ReadonlyFormFactory(
        type: ReadonlyFormType.numberRange,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        tip: currentFormItem.field.tip,
        value:
            DataClassUtils.dynamicToMapStringDynamic(value, defaultValue: null),
      );
    } else if (currentFormItem.field is ReadonlyPhoneNumberFF) {
      return ReadonlyFormFactory(
        type: ReadonlyFormType.phoneNumber,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToInt(value, defaultValue: null),
      );
    } else if (currentFormItem.field is ReadonlyAddressOnMapFF) {
      return ReadonlyFormFactory(
        type: ReadonlyFormType.addressOnMap,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToInt(value, defaultValue: null),
      );
    } else if (currentFormItem.field is ReadonlyDateFF) {
      return ReadonlyFormFactory(
        type: ReadonlyFormType.date,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToInt(value, defaultValue: null),
      );
    } else if (currentFormItem.field is ReadonlyDateTimeFF) {
      return ReadonlyFormFactory(
        type: ReadonlyFormType.dateTime,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToInt(value, defaultValue: null),
      );
    } else if (currentFormItem.field is ReadonlyTimeFF) {
      return ReadonlyFormFactory(
        type: ReadonlyFormType.time,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToInt(value, defaultValue: null),
      );
    } else if (currentFormItem.field is ReadonlyTextFF) {
      return ReadonlyFormFactory(
        type: ReadonlyFormType.text,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToString(value),
      );
    } else if (currentFormItem.field is ReadonlyTextListFF) {
      return ReadonlyFormFactory(
        type: ReadonlyFormType.textList,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToStringList(value),
      );
    } else if (currentFormItem.field is ReadonlyPhoneNumberListFF) {
      return ReadonlyFormFactory(
        type: ReadonlyFormType.phoneNumberList,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        tip: currentFormItem.field.tip,
        value: value,
      );
    } else if (currentFormItem.field is ReadonlyNumberFF) {
      return ReadonlyFormFactory(
        type: ReadonlyFormType.number,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        value: DataClassUtils.dynamicToString(value),
      );
    } else if (currentFormItem.field is TimeFF) {
      return TimeFormFactory(
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        state: state,
        value: DataClassUtils.dynamicToInt(value, defaultValue: null),
        tip: currentFormItem.field.tip,
        onChanged: onChanged,
      );
    } else if (currentFormItem.field is DateFF) {
      return DateFormFactory(
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        state: state,
        value: DataClassUtils.dynamicToInt(value, defaultValue: null),
        tip: currentFormItem.field.tip,
        onChanged: onChanged,
      );
    } else if (currentFormItem.field is TextListFF) {
      return TextListFormFactory(
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        onChanged: onChanged,
        state: state,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToStringList(value),
        type: ListType.text,
      );
    } else if (currentFormItem.field is EmailListFF) {
      return TextListFormFactory(
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        onChanged: onChanged,
        state: state,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToStringList(value),
        type: ListType.email,
      );
    } else if (currentFormItem.field is PhoneNumberListFF) {
      return PhoneNumberListFormFactory(
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        onChanged: onChanged,
        state: state,
        tip: currentFormItem.field.tip,
        value: value is List
            ? value
                .map((e) => DataClassUtils.dynamicToMapStringDynamic(e))
                .toList()
            : [],
      );
    } else if (currentFormItem.field is PhoneNumberFF) {
      return PhoneNumberFormFactory(
        onChanged: onChanged,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        state: state,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToMapStringDynamic(value),
      );
    } else if (currentFormItem.field is RichTextFF) {
      return RichTextFormFactory(
        onChanged: onChanged,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        state: state,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToString(value),
      );
    } else if (currentFormItem.field is PasswordFF) {
      return PasswordFormFactory(
        onChanged: onChanged,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        state: state,
        tip: currentFormItem.field.tip,
        value: DataClassUtils.dynamicToString(value, defaultValue: ""),
      );
    } else if (currentFormItem.field is ColorSelectorFF) {
      final currentValue =
          value ?? (currentFormItem.field as ColorSelectorFF).defaultValue;
      SelectorColors? colorSelectorValue;
      for (var color in SelectorColors.values) {
        if (color.name == currentValue) {
          colorSelectorValue = color;
        }
      }
      return ColorSelectorFormFactory(
        readOnly: (currentFormItem.field as ColorSelectorFF).readOnly,
        onChanged: onChanged,
        displayName: currentFormItem.field.displayName,
        name: currentFormItem.field.name,
        value: colorSelectorValue,
        tip: currentFormItem.field.tip,
        colors: (currentFormItem.field as ColorSelectorFF).colors,
      );
    } else if (currentFormItem.field is ParagraphHeaderFF) {
      return Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          sizedBoxH16,
          const Divider(
            thickness: 0.6,
          ),
          sizedBoxH32,
          SelectableText(
            currentFormItem.field.displayName,
            style: TextStyle(
              fontSize: 19,
              color: Colors.blue[700],
              fontWeight: FontWeight.bold,
            ),
          ),
        ],
      );
    } else if (currentFormItem.field is ParagraphFF) {
      return SizedBox(
        height: 90,
        child: Center(
          child: SelectableText(currentFormItem.field.displayName,
              style: const TextStyle(
                fontSize: 13,
              )),
        ),
      );
    }

    return Container();
  }
}
