// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import "package:flutter_multi_formatter/flutter_multi_formatter.dart";
import 'package:collection/collection.dart';
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class PhoneNumberListFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final Function(String name, dynamic value) onChanged;
  final List<Map<String, dynamic>>? value;
  final FormFieldState<dynamic>? state;
  final String? tip;

  const PhoneNumberListFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    this.value,
    this.state,
    this.tip,
  }) : super(key: key);

  @override
  State<PhoneNumberListFormFactory> createState() => _TextListState();
}

class _TextListState extends State<PhoneNumberListFormFactory> {
  List<PhoneNumberModel> _newValue = [];
  late StateSetter _setState;

  @override
  void initState() {
    super.initState();

    _newValue = (widget.value ?? [])
        .map(
          (e) => PhoneNumberModel.fromMap(e),
        )
        .toList();
  }

  @override
  void didUpdateWidget(covariant PhoneNumberListFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    final eq = const DeepCollectionEquality().equals;

    if (!eq(_newValue.map((e) => e.toMap()).toList(), widget.value)) {
      _newValue = (widget.value ?? [])
          .map(
            (e) => PhoneNumberModel.fromMap(e),
          )
          .toList();
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      tip: widget.tip,
      child: SelectableText(
        visualPhoneNumberList(
          _newValue.map((e) => e.toMap()).toList(),
          context: context,
        ).value,
      ),
      onPressed: () async {
        await showDialog(
          context: context,
          useSafeArea: true,
          builder: (BuildContext context) {
            return StatefulBuilder(builder: (BuildContext context, setState) {
              _setState = setState;

              return SimpleDialog(
                insetPadding: const EdgeInsets.all(15),
                contentPadding: const EdgeInsets.all(15),
                children: [
                  SizedBox(
                    height: 400,
                    width: 400,
                    child: Scaffold(
                      appBar: ThemedAppBar(
                        navigateBackButton: true,
                        title: widget.displayName,
                        actions: [
                          _AddTextFieldListItem(
                            onAddSubmit: (PhoneNumberModel item) {
                              _setState(() {
                                _newValue.add(item);
                              });
                            },
                          ),
                          ThemedIconButton(
                            icon: ThemedIcons.check(),
                            onPressed: () {
                              final phoneNumbers =
                                  _newValue.map((e) => e.toMap()).toList();

                              widget.onChanged(
                                widget.name,
                                phoneNumbers,
                              );
                              widget.state?.didChange(
                                phoneNumbers,
                              );
                              Navigator.of(context).pop();
                            },
                          ),
                        ],
                      ),
                      body: ListView.separated(
                        separatorBuilder: (context, index) => divider,
                        controller: ScrollController(),
                        itemCount: _newValue.length,
                        physics: const BouncingScrollPhysics(),
                        itemBuilder: (context, i) {
                          final item = _newValue[i];
                          return Padding(
                            padding: const EdgeInsets.all(5.0),
                            child: Row(
                              crossAxisAlignment: CrossAxisAlignment.center,
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: [
                                Expanded(
                                  child: _TextFieldListItem(
                                    value: item,
                                    onEditSubmit: (val) {
                                      _setState(() {
                                        _newValue = _newValue
                                            .asMap()
                                            .map(
                                              (key, value) => MapEntry(
                                                key,
                                                key == i ? val : value,
                                              ),
                                            )
                                            .values
                                            .toList();
                                      });
                                    },
                                  ),
                                ),
                                InkWell(
                                  onTap: () {
                                    _setState(() {
                                      _newValue.removeWhere((element) {
                                        return element == item;
                                      });
                                    });
                                  },
                                  child: ThemedCircleAvatar(
                                    backgroundColor: Colors.red,
                                    child: ThemedIcons.clear(
                                      iconStyle: IconStyle(
                                        color: Colors.white,
                                      ),
                                    ),
                                  ),
                                )
                              ],
                            ),
                          );
                        },
                      ),
                    ),
                  )
                ],
              );
            });
          },
        );
      },
    );
  }
}

class _TextFieldListItem extends StatefulWidget {
  final PhoneNumberModel value;
  final Function(PhoneNumberModel name) onEditSubmit;

  const _TextFieldListItem({
    Key? key,
    required this.value,
    required this.onEditSubmit,
  }) : super(key: key);

  @override
  _TextFieldListItemState createState() => _TextFieldListItemState();
}

class _TextFieldListItemState extends State<_TextFieldListItem> {
  PhoneNumberModel _value = PhoneNumberModel();
  PhoneNumberModel _newValue = PhoneNumberModel();
  late StateSetter _setState;

  @override
  void initState() {
    _value = widget.value;
    _newValue = _value;
    super.initState();
  }

  @override
  void didUpdateWidget(covariant _TextFieldListItem oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = widget.value;
      _newValue = _value;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.center,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        sizedBoxW8,
        SelectableText(
          visualPhoneNumber(_value.toMap(), context: context).value,
          style: const TextStyle(
            overflow: TextOverflow.fade,
          ),
        ),
        spacer,
        sizedBoxW8,
        ThemedCircleAvatar(
          child: Tooltip(
            message: "Call",
            child: ThemedIconButton(
                icon: ThemedIcons.smartphone(),
                onPressed: () async {
                  await launchUrl(
                    Uri.parse("tel://${_value.number}"),
                  );
                }),
          ),
        ),
        sizedBoxW4,
        ThemedCircleAvatar(
          child: Tooltip(
            message: "Edit",
            child: ThemedIconButton(
              icon: ThemedIcons.edit(),
              onPressed: () async {
                await showDialog(
                  context: context,
                  builder: (BuildContext context) {
                    return StatefulBuilder(
                      builder: (BuildContext context, setState) {
                        _setState = setState;
                        return AlertDialog(
                          contentPadding: const EdgeInsets.all(15),
                          insetPadding: const EdgeInsets.all(15),
                          actions: [
                            TextButton(
                              onPressed: () {
                                _setState(() {
                                  _newValue = _value;
                                });

                                Navigator.of(context).pop();
                              },
                              child: const Text("Cancel"),
                            ),
                            TextButton(
                              onPressed: () {
                                if (!isPhoneValid(_newValue.number ?? '')) {
                                  ThemedToast.error(
                                    context: context,
                                    message:
                                        "Please enter a valid phone number",
                                  );
                                } else {
                                  _setState(() {
                                    _value = _newValue;
                                  });
                                  widget.onEditSubmit(_newValue);

                                  Navigator.of(context).pop();
                                }
                              },
                              child: const Text("Save"),
                            ),
                          ],
                          content: Builder(
                            builder: (context) {
                              return SizedBox(
                                height: 150,
                                width: 400,
                                child: Scaffold(
                                  body: Builder(
                                    builder: (context) {
                                      return PhoneNumberFormFactory(
                                        displayName: "Phone Number",
                                        name: "phoneNumber",
                                        onChanged: (name, value) {
                                          _setState(() {
                                            _newValue =
                                                PhoneNumberModel.fromMap(
                                              DataClassUtils
                                                  .dynamicToMapStringDynamic(
                                                value,
                                              ),
                                            );
                                          });
                                        },
                                        value: _newValue.toMap(),
                                      );
                                    },
                                  ),
                                ),
                              );
                            },
                          ),
                        );
                      },
                    );
                  },
                );
              },
            ),
          ),
        ),
        sizedBoxW4,
      ],
    );
  }
}

class _AddTextFieldListItem extends StatelessWidget {
  final Function(PhoneNumberModel name) onAddSubmit;

  const _AddTextFieldListItem({
    Key? key,
    required this.onAddSubmit,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ThemedIconButton(
      onPressed: () async {
        await showDialog(
          context: context,
          builder: (BuildContext context) {
            return StatefulBuilder(
              builder: (BuildContext context, setState) {
                return _AddTextFieldListItemDialog(
                  onAddSubmit: onAddSubmit,
                );
              },
            );
          },
        );
      },
      icon: ThemedIcons.add(),
    );
  }
}

class _AddTextFieldListItemDialog extends StatefulWidget {
  final Function(PhoneNumberModel name) onAddSubmit;

  const _AddTextFieldListItemDialog({
    Key? key,
    required this.onAddSubmit,
  }) : super(key: key);

  @override
  _AddTextFieldListItemDialogState createState() =>
      _AddTextFieldListItemDialogState();
}

class _AddTextFieldListItemDialogState
    extends State<_AddTextFieldListItemDialog> {
  PhoneNumberModel _newLine = PhoneNumberModel();

  void _cleanState() {
    _newLine = PhoneNumberModel();
  }

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      contentPadding: const EdgeInsets.all(15),
      insetPadding: const EdgeInsets.all(15),
      actions: [
        TextButton(
          child: const Text("Cancel"),
          onPressed: () {
            _cleanState();
            setState(() {});
            Navigator.of(context).pop();
          },
        ),
        TextButton(
          child: const Text("Save"),
          onPressed: () {
            if (!isPhoneValid(_newLine.number ?? '')) {
              ThemedToast.error(
                context: context,
                message: "Please enter a valid phone number",
              );
            } else {
              widget.onAddSubmit(_newLine);
              setState(() {
                _newLine = PhoneNumberModel();
              });
              Navigator.of(context).pop();
            }

            return;
          },
        ),
      ],
      content: Builder(
        builder: (context) {
          return SizedBox(
            height: 150,
            width: 400,
            child: Scaffold(
              body: Builder(
                builder: (BuildContext context) {
                  return PhoneNumberFormFactory(
                    displayName: "Phone Number",
                    name: "phoneNumber",
                    onChanged: (name, value) {
                      setState(() {
                        _newLine = PhoneNumberModel.fromMap(
                          DataClassUtils.dynamicToMapStringDynamic(
                            value,
                          ),
                        );
                      });
                    },
                    value: _newLine.toMap(),
                  );
                },
              ),
            ),
          );
        },
      ),
    );
  }
}
