// 🎯 Dart imports:
import "dart:math";

// 🐦 Flutter imports:
import "package:flutter/material.dart";
import "package:flutter/services.dart";

// 📦 Package imports:
import "package:flutter_multi_formatter/flutter_multi_formatter.dart" as mf;
import "package:validators/validators.dart";

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

enum TextFieldInputFF { text, integer, email, double }

class TextFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final TextFieldInputFF type;
  final dynamic value;
  final FormFieldState<dynamic>? state;
  final bool showStandardBorder;
  final String? tip;

  /// in order to format a string like this GHJ45GHJHN to GHJ-45-GHJHN use String like this: "###-00-#####"
  final String? textFormatter;
  final Function(String name, dynamic value) onChanged;

  const TextFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.onChanged,
    this.showStandardBorder = true,
    this.type = TextFieldInputFF.text,
    this.value,
    this.textFormatter,
    this.state,
    this.tip,
  }) : super(key: key);
  @override
  State<TextFormFactory> createState() => _TextFormState();
}

class _TextFormState extends State<TextFormFactory> {
  TextEditingController? _controller;
  String _value = "";
  int _baseOffset = 0;

  @override
  void initState() {
    super.initState();

    _value = widget.value?.toString() ?? '';
    _controller = TextEditingController(
      text: _value,
    );

    _controller!.selection = TextSelection.fromPosition(
      TextPosition(
        offset: min(
          _baseOffset,
          _controller != null ? _controller!.text.length : 0,
        ),
      ),
    );

    _controller!.addListener(_listen);
  }

  @override
  void dispose() {
    _controller!.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant TextFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = widget.value?.toString() ?? '';
      _controller = TextEditingController(
        text: _value,
      );
      _controller!.selection = TextSelection.fromPosition(
        TextPosition(
          offset: min(
            _baseOffset,
            _controller != null ? _controller!.text.length : 0,
          ),
        ),
      );
      _controller!.addListener(_listen);
    }
  }

  void _listen() {
    // setState(() {
    _baseOffset = _controller!.selection.baseOffset;
    // });
  }

  List<TextInputFormatter>? _getFormatter() {
    if (widget.type == TextFieldInputFF.double) {
      return <TextInputFormatter>[
        TextInputFormatter.withFunction((oldValue, newValue) {
          try {
            String text = newValue.text;
            if (text.isEmpty) {
              return newValue;
            }
            if (text.endsWith('.')) {
              text += text.replaceFirst('.', '');
            }
            double.parse(text);
            return newValue;
          } catch (e) {
            return oldValue;
          }
        }),
      ];
    }
    if (widget.type == TextFieldInputFF.integer) {
      return <TextInputFormatter>[
        TextInputFormatter.withFunction((oldValue, newValue) {
          try {
            String text = newValue.text;
            if (text.isEmpty) {
              return newValue;
            }

            int.parse(text);
            return newValue;
          } catch (e) {
            return oldValue;
          }
        }),
      ];
    }
    return [
      if (widget.textFormatter != null)
        mf.MaskedInputFormatter(widget.textFormatter!)
    ];
  }

  Widget _form() {
    return TextFormField(
      cursorColor: Colors.amberAccent,
      onChanged: (val) {
        if (widget.type == TextFieldInputFF.integer) {
          int? integerValue = int.tryParse(val);
          widget.onChanged(widget.name, integerValue);
          widget.state?.didChange(integerValue);
        } else if (widget.type == TextFieldInputFF.double) {
          double? doubleValue = double.tryParse(
            val.endsWith('.') ? "${val}0" : val,
          );
          widget.onChanged(widget.name, doubleValue);
          widget.state?.didChange(doubleValue);
        } else if (widget.type == TextFieldInputFF.text ||
            widget.type == TextFieldInputFF.email) {
          widget.onChanged(widget.name, val);
          widget.state?.didChange(val);
        }
      },
      validator: (value) {
        if (widget.type == TextFieldInputFF.email &&
            value != null &&
            value != "" &&
            !isEmail(value)) {
          return "Please insert a valid email";
        }
        return null;
      },
      controller: _controller,
      keyboardType: TextInputType.text,
      inputFormatters: _getFormatter(),
      textDirection: TextDirection.ltr,
      autofocus: false,
      decoration: InputDecoration(
        label: widget.showStandardBorder ? null : Text(widget.displayName),
        suffixIcon: ThemedIconButton(
          onPressed: () {
            _controller!.clear();
            widget.onChanged(widget.name, "");
            widget.state?.didChange("");
          },
          icon: ThemedIcons.clear(
            iconStyle: IconStyle(
              color: Colors.redAccent,
            ),
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (context, constraints) {
      return SizedBox(
        width: constraints.maxWidth,
        child: widget.showStandardBorder
            ? BorderLabel(
                label: widget.displayName,
                tip: widget.tip,
                child: _form(),
              )
            : _form(),
      );
    });
  }
}
