// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:collection/collection.dart';
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import 'package:verttice_ui/src/const/box_sizes.dart';
import 'package:verttice_ui/src/const/icons.dart';
import 'package:verttice_ui/src/widgets/data_factory/form_factory/form_factory.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

import '../data_factory_index.dart';

class FilterView extends StatefulWidget {
  final Function(List<FilterField>) filterCallback;
  final Function() clearFilterCallback;
  final List<FilterField>? filters;
  final FormItem column;

  const FilterView({
    Key? key,
    required this.filterCallback,
    required this.clearFilterCallback,
    required this.column,
    this.filters,
  }) : super(key: key);

  @override
  State<FilterView> createState() => _FiltersViewState();
}

class _FiltersViewState extends State<FilterView> {
  late List<FilterField> _filters;
  bool _exactMatch = false;

  @override
  void initState() {
    super.initState();
    if (widget.filters == null || widget.filters!.isEmpty) {
      _filters = [
        FilterField(
          field: widget.column.field.fullSearchPath,
          filterType: widget.column.field.filterType,
        )
      ];
    } else {
      bool allMarkedAsExactMatch = true;
      for (var filter in widget.filters!) {
        if (!filter.exactMatch) {
          allMarkedAsExactMatch = false;
        }
      }
      _exactMatch = allMarkedAsExactMatch;
      _filters = widget.filters!;
    }
  }

  @override
  void didUpdateWidget(covariant FilterView oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_filters, widget.filters)) {
      if (widget.filters == null || widget.filters!.isEmpty) {
        _filters = [
          FilterField(
            field: widget.column.field.fullSearchPath,
            filterType: widget.column.field.filterType,
          )
        ];
      } else {
        bool allMarkedAsExactMatch = true;
        for (var filter in widget.filters!) {
          if (!filter.exactMatch) {
            allMarkedAsExactMatch = false;
          }
        }
        _exactMatch = allMarkedAsExactMatch;
        _filters = widget.filters!;
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        ThemedAppBar(
          navigateBackButton: true,
          actions: [
            if (widget.column.field.filterType == FilterTypesFF.string ||
                widget.column.field.filterType == FilterTypesFF.stringList)
              Tooltip(
                message: 'Add filter',
                child: ThemedIconButton(
                  onPressed: () {
                    setState(() {
                      _filters.add(
                        FilterField(
                          field: widget.column.field.fullSearchPath,
                          filterType: widget.column.field.filterType,
                        ),
                      );
                    });
                  },
                  icon: ThemedIcons.add(),
                ),
              )
          ],
        ),
        Expanded(
          child: !(widget.column.field.filterType == FilterTypesFF.string ||
                  widget.column.field.filterType == FilterTypesFF.stringList)
              ? Center(
                  child: FormFactory(
                    returnFormStateOnLoad: true,
                    formItems: [widget.column],
                    gridView: true,
                    isFilter: true,
                    formState: {widget.column.field.name: _filters.first.value},
                    onChanged: (name, data) {
                      setState(() {
                        _filters.first = FilterField(
                          field: widget.column.field.fullSearchPath,
                          filterType: widget.column.field.filterType,
                          value: data,
                        );
                      });
                    },
                  ),
                )
              : ListView(
                  controller: ScrollController(),
                  children: _filters
                      .mapIndexed(
                        (index, e) => Row(
                          crossAxisAlignment: CrossAxisAlignment.center,
                          mainAxisAlignment: MainAxisAlignment.spaceAround,
                          children: [
                            Expanded(
                              child: FormFactory(
                                returnFormStateOnLoad: true,
                                formItems: [widget.column],
                                gridView: true,
                                isFilter: true,
                                formState: {
                                  widget.column.field.name: e.value,
                                },
                                onChanged: (name, data) {
                                  setState(() {
                                    _filters[index] = FilterField(
                                      field: widget.column.field.fullSearchPath,
                                      filterType:
                                          widget.column.field.filterType,
                                      value: data,
                                    );
                                  });
                                },
                              ),
                            ),
                            if (_filters.length > 1)
                              Padding(
                                padding: const EdgeInsets.all(12.0),
                                child: Tooltip(
                                  message: 'Remove this filter',
                                  child: ThemedIconButton(
                                    onPressed: () {
                                      setState(() {
                                        _filters.removeAt(index);
                                      });
                                    },
                                    icon: ThemedIcons.clear(),
                                  ),
                                ),
                              )
                          ],
                        ),
                      )
                      .toList(),
                ),
        ),
        if (widget.column.field.filterType == FilterTypesFF.string ||
            widget.column.field.filterType == FilterTypesFF.stringList)
          SizedBox(
            height: 50,
            child: CheckboxListTile(
              value: _exactMatch,
              title: const Text('Exact Match?'),
              onChanged: (bool? value) {
                setState(() {
                  _exactMatch = value ?? false;
                });
              },
            ),
          ),
        SizedBox(
          height: 50,
          child: Row(children: [
            sizedBoxW8,
            TextButton(
              onPressed: () {
                widget.clearFilterCallback();
                Navigator.of(context).pop();
              },
              child: const Text(
                'Clear Filter',
                style: TextStyle(
                  color: Colors.red,
                ),
              ),
            ),
            sizedBoxW8,
            TextButton(
              onPressed: () {
                widget.filterCallback(_filters.map((e) {
                  final currentFilter = e;
                  currentFilter.setExactMatch(_exactMatch);
                  return currentFilter;
                }).toList());
                Navigator.of(context).pop();
              },
              child: const Text(
                'Apply Filter',
                style: TextStyle(
                  color: Colors.green,
                ),
              ),
            ),
            sizedBoxW8,
            Tooltip(
              message: "Cancel",
              child: TextButton(
                onPressed: () {
                  Navigator.of(context).pop();
                },
                child: const Text("Cancel"),
              ),
            ),
            sizedBoxW8,
          ]),
        )
      ],
    );
  }
}
