// 🐦 Flutter imports:
import "package:flutter/material.dart";
import "package:flutter/scheduler.dart";
import "package:flutter/services.dart";

// 📦 Package imports:
import "package:collection/collection.dart";
import "package:provider/provider.dart";
import 'package:syncfusion_flutter_datagrid/datagrid.dart';
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class TableContent extends StatefulWidget {
  final bool enableCopyButton;
  final List<FormItem> columns;
  final List<Map<String, dynamic>> data;
  final void Function(
    Map<String, dynamic>? rowData,
    int rowIndex,
  ) handleSelectedRow;
  final void Function(List<Map<String, dynamic>>)? selectedListCallback;
  final Function(Map<String, List<FilterField>>) onFilterChanged;
  final Map<String, List<FilterField>> filters;
  final String uniqueIdField;
  final void Function({
    required String field,
    required bool asc,
  })? onSortChanged;

  const TableContent({
    Key? key,
    required this.enableCopyButton,
    required this.columns,
    required this.data,
    required this.handleSelectedRow,
    required this.filters,
    required this.onFilterChanged,
    required this.uniqueIdField,
    this.onSortChanged,
    this.selectedListCallback,
  }) : super(key: key);

  @override
  State<TableContent> createState() => TableContentState();
}

class TableContentState extends State<TableContent> {
  final GlobalKey<SfDataGridState> dataGridKey = GlobalKey<SfDataGridState>();
  final List<FormItem> _columns = [];
  late List<Map<String, dynamic>> _data;
  late Map<String, List<FilterField>> _filters;
  final List<Map<String, dynamic>> _selectedRows = [];
  int _additionalColumns = 0;
  String? _selectedRow;
  int _currentSortColumn = 0;
  bool _isAscending = true;
  DataGridController _controller = DataGridController();

  @override
  void initState() {
    super.initState();
    _filters = widget.filters;
    _data = widget.data;
    _columns.add(
      FormItem(
        field: NoneFF(
          displayName: "Index",
          name: "index",
          tableColumnWidth: 25,
        ),
      ),
    );
    _additionalColumns += 1;
    _columns.add(
      FormItem(
        field: NoneFF(
          displayName: "Copy",
          name: "copy",
          tableColumnWidth: 35,
        ),
      ),
    );
    _additionalColumns += 1;
    if (widget.selectedListCallback != null) {
      _additionalColumns += 1;
      _columns.add(
        FormItem(
          field: NoneFF(
            displayName: "Select",
            name: "select",
            tableColumnWidth: 40,
          ),
        ),
      );
    }
    _columns.addAll(
      widget.columns
          .where(
            (item) =>
                item.field.showOnTable &&
                item.field.displayName.isNotEmpty &&
                item.field.name.isNotEmpty,
          )
          .toList(),
    );
  }

  @override
  void didUpdateWidget(covariant TableContent oldWidget) {
    super.didUpdateWidget(oldWidget);
    Function unOrdDeepEq = const DeepCollectionEquality.unordered().equals;
    if (!unOrdDeepEq(_data, widget.data)) {
      _data = widget.data;
      _selectedRows.clear();
    }

    if (!const MapEquality().equals(_filters, widget.filters)) {
      _filters = widget.filters;
    }
  }

  @override
  void setState(VoidCallback fn) {
    if (mounted) {
      SchedulerBinding.instance.addPostFrameCallback((_) {
        super.setState(fn);
      });
    }
  }

  List<GridColumn> _buildColumns({
    required Map<String, List<FilterField>> filters,
  }) {
    final List<GridColumn> tableColumns = _columns.mapIndexed((index, column) {
      if (index == 0) {
        return GridColumn(
          width: column.field.tableColumnWidth,
          columnName: column.field.displayName,
          label: const FittedBox(
            fit: BoxFit.scaleDown,
            child: Icon(Icons.numbers),
          ),
        );
      } else if (index == 1) {
        return GridColumn(
          width: column.field.tableColumnWidth,
          columnName: column.field.displayName,
          label: const Center(
            child: Text(
              "Copy",
              style: TextStyle(
                overflow: TextOverflow.fade,
                fontSize: 11,
              ),
              softWrap: true,
            ),
          ),
        );
      } else if (index == 2 && widget.selectedListCallback != null) {
        return GridColumn(
          width: column.field.tableColumnWidth,
          columnName: column.field.displayName,
          label: const Center(
            child: Text(
              "Select",
              style: TextStyle(
                overflow: TextOverflow.fade,
                fontSize: 11,
              ),
              softWrap: true,
            ),
          ),
        );
      }
      return _buildDataColumn(
        column: column,
        filters: filters,
        columnIndex: index,
      );
    }).toList();

    return tableColumns;
  }

  GridColumn _buildDataColumn({
    required FormItem column,
    required Map<String, List<FilterField>> filters,
    required int columnIndex,
  }) {
    return GridColumn(
      minimumWidth: column.field.tableColumnWidth,
      columnWidthMode: ColumnWidthMode.fill,
      columnName: column.field.displayName,
      allowSorting: false,
      label: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          sizedBoxH2,
          InkWell(
            onTap: () {
              if (column.field.filterType == FilterTypesFF.number ||
                  column.field.filterType == FilterTypesFF.numberRange ||
                  column.field.filterType == FilterTypesFF.date ||
                  column.field.filterType == FilterTypesFF.dateRange ||
                  column.field.filterType == FilterTypesFF.time ||
                  column.field.filterType == FilterTypesFF.timeRange ||
                  column.field.filterType == FilterTypesFF.string ||
                  column.field.filterType == FilterTypesFF.stringList) {
                setState(() {
                  _isAscending = _currentSortColumn == columnIndex
                      ? !_isAscending
                      : _isAscending;

                  _currentSortColumn = columnIndex;

                  if (widget.onSortChanged != null) {
                    widget.onSortChanged!(
                      field: _columns[columnIndex].field is DateRangeFF
                          ? "${_columns[columnIndex].field.name}.startDate"
                          : _columns[columnIndex].field.name,
                      asc: _isAscending,
                    );
                  } else {
                    _handleSortData(
                      data: _data,
                      column: column,
                      name: column.field.name,
                      desc: !_isAscending,
                    );
                  }
                });
              }
            },
            child: SizedBox(
              height: 35,
              child: Row(
                children: [
                  SizedBox(
                    width: _columns[columnIndex].field.tableColumnWidth - 25,
                    child: Text(
                      column.field.displayName,
                      style: const TextStyle(
                        overflow: TextOverflow.fade,
                        fontSize: 11,
                      ),
                      softWrap: true,
                    ),
                  ),
                  spacer,
                  if (_currentSortColumn == columnIndex && _isAscending)
                    SizedBox(
                      width: 25,
                      child: ThemedIcons.arrowDown(),
                    )
                  else if (_currentSortColumn == columnIndex && !_isAscending)
                    SizedBox(
                      width: 25,
                      child: ThemedIcons.arrowUp(),
                    )
                ],
              ),
            ),
          ),
          SizedBox(
            height: 30,
            child: Builder(builder: (context) {
              String? labelText = filters[column.field.name]
                  ?.map(
                    (e) => visualDataProcessor(
                      item: column,
                      row: {
                        column.field.name: e.value,
                      },
                      isFilter: true,
                      context: context,
                    ).value,
                  )
                  .toList()
                  .join(", ");
              if (column.field is CustomFF &&
                  (column.field as CustomFF).tableFilterButtonLabelBuilder !=
                      null) {
                labelText =
                    (column.field as CustomFF).tableFilterButtonLabelBuilder!(
                  context: context,
                  filters: filters[column.field.name] ?? [],
                );
              }

              return Tooltip(
                message:
                    labelText != null ? "Filter: $labelText" : "Apply filter",
                child: ThemedCleanTextButton(
                  labelText ?? '',
                  textStyle: const TextStyle(
                    fontSize: 12,
                  ),
                  icon: ThemedIcons.filter(
                    iconStyle: IconStyle(
                      size: 17,
                    ),
                  ),
                  onPressed: () => _filterView(
                    column: column,
                    filters: filters,
                  ),
                ),
              );
            }),
          ),
        ],
      ),
    );
  }

  Future<void> _filterView({
    required FormItem column,
    required Map<String, List<FilterField>> filters,
  }) async {
    if (column.field is CustomFF &&
        (column.field as CustomFF).tableFilterDialog != null) {
      await (column.field as CustomFF).tableFilterDialog!(
        context: context,
        clearFilterCallback: () {
          final updatedFilters = filters;
          updatedFilters.remove(column.field.name);
          widget.onFilterChanged(updatedFilters);
        },
        filterCallback: (List<FilterField> filter) {
          final updatedFilters = filters;
          updatedFilters[column.field.name] = filter;
          widget.onFilterChanged(updatedFilters);
        },
        filters: filters[column.field.name],
      );
      return;
    }

    await sizedDialog(
      context: context,
      height: 450,
      width: 400,
      builder: (context) {
        return FilterView(
          column: column,
          filters: filters[column.field.name],
          clearFilterCallback: () {
            final updatedFilters = filters;
            updatedFilters.remove(column.field.name);
            widget.onFilterChanged(updatedFilters);
          },
          filterCallback: (List<FilterField> filter) {
            final updatedFilters = filters;
            updatedFilters[column.field.name] = filter;
            widget.onFilterChanged(updatedFilters);
          },
        );
      },
    );
  }

  DataGridRow _dataRowBuilder(
    int rowIndex,
    Map<String, dynamic> rowData,
  ) {
    List<DataGridCell> currentRow =
        _columns.mapIndexed<DataGridCell>((index, column) {
      if (index == 0) {
        return DataGridCell<int>(
          columnName: column.field.displayName,
          value: rowIndex + 1,
        );
      } else if (index == 1) {
        return DataGridCell<String>(
          columnName: column.field.displayName,
          value: "Copy",
        );
      } else if (index == 2 && widget.selectedListCallback != null) {
        return DataGridCell<Widget>(
          columnName: column.field.displayName,
          value: Checkbox(
            value: _selectedRows.contains(rowData),
            onChanged: (bool? checked) {
              if (checked != null && checked) {
                _selectedRows.add(rowData);
              } else {
                _selectedRows.remove(rowData);
              }
              widget.selectedListCallback!(_selectedRows);
              setState(() {});
            },
          ),
        );
      }
      final visualData = visualDataProcessor(
        item: column,
        row: rowData,
        context: context,
      );

      return DataGridCell<dynamic>(
        columnName: column.field.displayName,
        value: visualData.value,
      );
    }).toList();

    return DataGridRow(
      cells: currentRow,
    );
  }

  @override
  Widget build(BuildContext context) {
    return Consumer<VertticeUIState>(builder: (context, value, _) {
      return SfDataGrid(
        key: dataGridKey,
        onCellTap: (DataGridCellTapDetails dataGridCellTapDetails) {
          final rowIndex = dataGridCellTapDetails.rowColumnIndex.rowIndex - 1;
          final rowData = _data[rowIndex];
          widget.handleSelectedRow(rowData, rowIndex);

          setState(() {
            _selectedRow = rowData[widget.uniqueIdField];
          });
        },
        columnSizer: ColumnSizer(),
        allowSwiping: true,
        frozenColumnsCount: _additionalColumns,
        headerRowHeight: 70,
        source: DataGridDataSource(
          isDarkTheme: value.isDarkTheme,
          rowsData: _data,
          selectedRow: _selectedRow,
          uniqueIdField: widget.uniqueIdField,
          columns: _columns,
          context: context,
          data: _data.mapIndexed((rowIndex, rowData) {
            return _dataRowBuilder(
              rowIndex,
              rowData,
            );
          }).toList(),
        ),
        // showCheckboxColumn: widget.selectedListCallback != null,
        // onSelectionChanged: widget.selectedListCallback != null
        //     ? (
        //         List<DataGridRow> addedRows,
        //         List<DataGridRow> removedRows,
        //       ) {
        //         setState(() {
        //           for (var dataGridRow in addedRows) {
        //             _selectedRows.add(dataGridRow);
        //             _selectedItems.add(
        //               _data[dataGridRow.getCells()[0].value - 1],
        //             );
        //           }

        //           for (var dataGridRow in removedRows) {
        //             _selectedRows.remove(dataGridRow);

        //             _selectedItems.remove(
        //               _data[dataGridRow.getCells()[0].value - 1],
        //             );
        //           }
        //           _controller = DataGridController(
        //             selectedRows: _selectedRows,
        //           );
        //         });
        //         widget.selectedListCallback!(_selectedItems);
        //       }
        //     : null,
        selectionMode: SelectionMode.single,
        controller: _controller,
        gridLinesVisibility: GridLinesVisibility.both,
        verticalScrollController: ScrollController(),
        navigationMode: GridNavigationMode.row,
        isScrollbarAlwaysShown: true,
        columns: _buildColumns(
          filters: _filters,
        ),
      );
    });
  }
}

class DataGridDataSource extends DataGridSource {
  final List<Map<String, dynamic>> rowsData;
  final String uniqueIdField;
  final String? selectedRow;
  final bool isDarkTheme;
  final BuildContext context;
  final List<FormItem> columns;

  DataGridDataSource({
    required List<DataGridRow> data,
    required this.rowsData,
    required this.uniqueIdField,
    required this.selectedRow,
    required this.isDarkTheme,
    required this.context,
    required this.columns,
  }) {
    _data = data;
  }

  List<DataGridRow> _data = [];

  @override
  List<DataGridRow> get rows => _data;

  @override
  DataGridRowAdapter? buildRow(DataGridRow row) {
    final cells = row.getCells();
    final rowIndex = cells.first.value - 1;
    final rowData = rowsData[cells.first.value - 1];
    final rowID = rowData[uniqueIdField];

    return DataGridRowAdapter(
      color: selectedRow == rowID && rowID != null
          ? isDarkTheme
              ? Colors.black
              : Colors.grey[400]
          : ((rowIndex % 2) == 0)
              ? isDarkTheme
                  ? Colors.grey[600]
                  : Colors.grey[50]
              : isDarkTheme
                  ? Colors.grey[800]
                  : Colors.grey[200],
      cells: row.getCells().mapIndexed<Widget>((int cellIndex, dataGridCell) {
        final column = columns.firstWhere(
          (element) => element.field.displayName == dataGridCell.columnName,
        );
        final visualData = visualDataProcessor(
          item: column,
          row: rowData,
          context: context,
        );

        if (cellIndex == 0) {
          return _CellWidget(
            child: FittedBox(
              fit: BoxFit.scaleDown,
              child: Text(
                (rowIndex + 1).toString(),
              ),
            ),
          );
        } else if (cellIndex == 1) {
          return _CellWidget(
            child: CopyTableRow(
              columns: columns,
              rowData: rowData,
            ),
          );
        } else if (dataGridCell.value is Widget) {
          return dataGridCell.value;
        }
        return _CellWidget(
          child: visualData.widget,
        );
      }).toList(),
    );
  }
}

class CopyTableRow extends StatelessWidget {
  final List<FormItem> columns;
  final Map<String, dynamic> rowData;

  const CopyTableRow({
    Key? key,
    required this.columns,
    required this.rowData,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Tooltip(
      message: "Copy row",
      child: ThemedIconButton(
        onPressed: () async {
          await Clipboard.setData(
            ClipboardData(
              text: columns
                  .map(
                    (e) => "${e.field.displayName}: ${visualDataProcessor(
                      item: e,
                      row: rowData,
                      context: context,
                    ).value}",
                  )
                  .toList()
                  .join(',\n'),
            ),
          );
          ThemedToast.info(
            context: context,
            message: "Row copied to clipboard",
          );
        },
        icon: ThemedIcons.copy(),
      ),
    );
  }
}

class _CellWidget extends StatelessWidget {
  final Widget child;
  const _CellWidget({
    Key? key,
    required this.child,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Stack(children: [
      TransparentPointer(
        child: Container(
          alignment: Alignment.centerLeft,
          padding: const EdgeInsets.all(2.0),
          child: child,
        ),
      ),
    ]);
  }
}

void _handleSortData({
  required List<Map<String, dynamic>> data,
  required FormItem column,
  required String name,
  required bool desc,
}) {
  data.sort((a, b) {
    dynamic aS = a[name] ?? "";
    dynamic bS = b[name] ?? "";
    if (column.field.dataBuilder != null) {
      aS = column.field.dataBuilder!(
        a.cast<String, dynamic>(),
      );
      bS = column.field.dataBuilder!(
        b.cast<String, dynamic>(),
      );
    }

    if (aS == null || aS == "") {
      if (bS == null || bS == "") {
        return 0;
      } else {
        return 1;
      }
    } else {
      if (bS == null || bS == "") {
        return -1;
      }
    }

    var r = desc
        ? bS.toString().compareTo(aS.toString())
        : aS.toString().compareTo(bS.toString());
    if (r != 0) return r;
    return desc
        ? bS.toString().compareTo(aS.toString())
        : aS.toString().compareTo(bS.toString());
  });
}
