// 🎯 Dart imports:
import "dart:math";

// 🐦 Flutter imports:
import "package:flutter/material.dart";
import "package:flutter/scheduler.dart";

// 📦 Package imports:
import "package:collection/collection.dart";

// 🌎 Project imports:
import '../data_factory_index.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';
import 'table_frame.dart';

class TableFactory extends StatefulWidget {
  final List<Map<String, dynamic>> data;
  final List<FormItem> columns;
  final String title;
  final void Function(Map<String, dynamic>? rowData)? rowPressCallback;
  final String uniqueIdField;
  final Widget? topBarWidget;
  final double? topBarPercentageHeight;
  final double actionsBarHeight;
  final bool showCopyRowButton;
  final List<Widget>? tableActions;
  final void Function(List<Map<String, dynamic>>)? selectedListCallback;

  const TableFactory({
    Key? key,
    required this.data,
    required this.columns,
    required this.uniqueIdField,
    this.title = "Table",
    this.rowPressCallback,
    this.showCopyRowButton = true,
    this.topBarWidget,
    this.topBarPercentageHeight,
    this.tableActions,
    this.selectedListCallback,
    this.actionsBarHeight = 48,
  }) : super(key: key);

  @override
  State<TableFactory> createState() => _TableFactoryState();
}

class _TableFactoryState extends State<TableFactory> {
  final double _separator = 2.0;
  late List<Map<String, dynamic>> _data;
  late double _topHeightPercentage;

  @override
  void initState() {
    super.initState();
    _data = widget.data;
    _topHeightPercentage = (widget.topBarWidget != null)
        ? min(widget.topBarPercentageHeight ?? 0.4, 0.4)
        : 0.0;
  }

  @override
  void setState(VoidCallback fn) {
    if (mounted) {
      SchedulerBinding.instance.addPostFrameCallback((_) {
        super.setState(fn);
      });
    }
  }

  @override
  void didUpdateWidget(covariant TableFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_data, widget.data)) {
      _data = widget.data;
    }
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (context, constraints) {
        final double topPanelHeight =
            (constraints.maxHeight * _topHeightPercentage);
        return Stack(
          children: [
            SizedBox(
              height: constraints.maxHeight,
              width: constraints.maxWidth,
            ),
            AnimatedPositioned(
              duration: const Duration(milliseconds: 200),
              top: 0,
              child: (widget.topBarWidget != null)
                  ? SizedBox(
                      height: topPanelHeight,
                      width: constraints.maxWidth,
                      child: widget.topBarWidget,
                    )
                  : const SizedBox(
                      height: 0,
                      width: 0,
                    ),
            ),
            AnimatedPositioned(
              duration: const Duration(milliseconds: 200),
              top: topPanelHeight,
              height: _separator,
              width: constraints.maxWidth,
              child: MouseRegion(
                cursor: SystemMouseCursors.resizeRow,
                child: GestureDetector(
                  onPanUpdate: (DragUpdateDetails dragDetail) {
                    setState(() {
                      _topHeightPercentage =
                          dragDetail.globalPosition.dy / constraints.maxHeight;
                    });
                  },
                  child: ThemedDivider(
                    height: _separator,
                    width: constraints.maxWidth,
                  ),
                ),
              ),
            ),
            AnimatedPositioned(
              duration: const Duration(milliseconds: 300),
              top: topPanelHeight + _separator,
              height: constraints.maxHeight - (topPanelHeight + _separator),
              width: constraints.maxWidth,
              child: TableFrame(
                data: _data,
                actionsBarHeight: widget.actionsBarHeight,
                uniqueIdField: widget.uniqueIdField,
                tableActions: widget.tableActions,
                frameWidth: constraints.maxWidth,
                frameHeight:
                    constraints.maxHeight - (topPanelHeight + _separator),
                enableCopyButton: widget.showCopyRowButton,
                selectedListCallback: widget.selectedListCallback,
                columns: widget.columns,
                title: widget.title,
                handleSelectedRow:
                    (Map<String, dynamic>? rowData, int rowIndex) {
                  if (widget.rowPressCallback != null) {
                    widget.rowPressCallback!(rowData);
                  }
                },
              ),
            )
          ],
        );
      },
    );
  }
}
