// 🐦 Flutter imports:
import "package:flutter/material.dart";
import "package:flutter/scheduler.dart";

// 📦 Package imports:
import "package:collection/collection.dart";
import 'package:syncfusion_flutter_datagrid_export/export.dart';
import 'package:syncfusion_flutter_pdf/pdf.dart';
import 'package:syncfusion_flutter_xlsio/xlsio.dart' hide Column, Row;
import 'package:verttice_models/verttice_models.dart';

// 🌎 Project imports:
import '../data_factory_index.dart';
import 'package:verttice_ui/src/const/box_sizes.dart';
import 'package:verttice_ui/src/const/icons.dart';
import 'package:verttice_ui/src/widgets/fade_edges.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';
import 'table_content.dart';

import 'package:verttice_ui/src/utils/file_saver/save_file_mobile.dart'
    if (dart.library.html) 'package:verttice_ui/src/utils/file_saver/save_file_web.dart'
    as helper;

class TableFrameServer extends StatefulWidget {
  final bool enableCopyButton;
  final void Function(List<Map<String, dynamic>>)? selectedListCallback;
  final List<FormItem> columns;
  final List<Map<String, dynamic>> data;
  final double frameWidth;
  final double frameHeight;
  final double? actionPercentageWidth;
  final void Function(
    Map<String, dynamic>? rowData,
    int rowIndex,
  ) handleSelectedRow;
  final List<Widget>? tableActions;
  final String title;

  final Function(Map<String, List<FilterField>>) onFilterChanged;
  final Function(String)? onSearchChanged;
  final String uniqueIdField;
  final void Function({
    required String field,
    required bool asc,
  }) onSortChanged;

  const TableFrameServer({
    Key? key,
    required this.enableCopyButton,
    required this.selectedListCallback,
    required this.columns,
    required this.data,
    required this.frameWidth,
    required this.frameHeight,
    required this.handleSelectedRow,
    required this.title,
    required this.onFilterChanged,
    required this.uniqueIdField,
    required this.onSortChanged,
    this.onSearchChanged,
    this.tableActions,
    this.actionPercentageWidth,
  }) : super(key: key);

  @override
  State<TableFrameServer> createState() => _TableFrameState();
}

class _TableFrameState extends State<TableFrameServer> {
  final GlobalKey<TableContentState> _dataTableContentKey =
      GlobalKey<TableContentState>();
  final _debouncer = Debouncer(milliseconds: 500);
  final double _topBarHeight = 40;
  final double _separator = 2;
  late List<Map<String, dynamic>> _data;
  String _search = "";
  Map<String, List<FilterField>> _filters = {};

  @override
  void initState() {
    _data = widget.data;
    super.initState();
  }

  @override
  void didUpdateWidget(covariant TableFrameServer oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_data, widget.data)) {
      _data = widget.data;
    }
  }

  @override
  void setState(VoidCallback fn) {
    if (mounted) {
      SchedulerBinding.instance.addPostFrameCallback((_) {
        super.setState(fn);
      });
    }
  }

  Future<void> _exportDataGridToExcel() async {
    final Workbook workbook = _dataTableContentKey
        .currentState!.dataGridKey.currentState!
        .exportToExcelWorkbook();

    final List<int> bytes = workbook.saveAsStream();
    workbook.dispose();

    await helper.saveAndLaunchFile(bytes, '${widget.title}DataGrid.xlsx');
  }

  Future<void> _exportDataGridToPdf() async {
    final PdfDocument document = _dataTableContentKey
        .currentState!.dataGridKey.currentState!
        .exportToPdfDocument(
      fitAllColumnsInOnePage: false,
      autoColumnWidth: true,
    );

    final List<int> bytes = await document.save();
    await helper.saveAndLaunchFile(bytes, '${widget.title}Table.pdf');
    document.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: widget.frameWidth,
      height: widget.frameHeight,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          SizedBox(
            width: widget.frameWidth,
            height: _topBarHeight,
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Container(
                  width:
                      widget.frameWidth / (widget.tableActions != null ? 2 : 1),
                  padding: const EdgeInsets.symmetric(horizontal: 5),
                  child: ListView(
                    controller: ScrollController(),
                    scrollDirection: Axis.horizontal,
                    children: [
                      sizedBoxW16,
                      Center(
                        child: SelectableText(
                          widget.title,
                          style: const TextStyle(
                            fontWeight: FontWeight.bold,
                            fontSize: 20,
                          ),
                        ),
                      ),
                      sizedBoxW32,
                      Tooltip(
                        message: "Clear all filters",
                        child: ThemedIconButton(
                          onPressed: () {
                            setState(() {
                              _search = '';
                              _filters = {};
                              widget.onFilterChanged({});
                              if (widget.onSearchChanged != null) {
                                widget.onSearchChanged!('');
                              }
                            });
                          },
                          icon: ThemedIcons.clear(),
                        ),
                      ),
                      sizedBoxW8,
                      Tooltip(
                        message: 'Export to Pdf',
                        child: ThemedIconButton(
                          onPressed: _exportDataGridToPdf,
                          icon: ThemedIcons.pdfFile(),
                        ),
                      ),
                      sizedBoxW8,
                      Tooltip(
                        message: 'Export to Excel',
                        child: ThemedIconButton(
                          onPressed: _exportDataGridToExcel,
                          icon: ThemedIcons.sheet(),
                        ),
                      ),
                      sizedBoxW8,
                      if (widget.onSearchChanged != null)
                        FittedBox(
                          fit: BoxFit.scaleDown,
                          child: SizedBox(
                            width: 170,
                            child: TableSearchBar(
                              changeSearch: (String search) {
                                _debouncer.run(() {
                                  setState(() {
                                    _search = search;
                                    widget.onSearchChanged!(_search);
                                  });
                                });
                              },
                            ),
                          ),
                        ),
                      sizedBoxW8,
                    ],
                  ),
                ),
                if (widget.tableActions != null)
                  Container(
                    padding: const EdgeInsets.symmetric(horizontal: 5),
                    width: widget.frameWidth / 2,
                    child: FadeEdges(
                      child: ListView(
                        controller: ScrollController(),
                        scrollDirection: Axis.horizontal,
                        children: [
                          sizedBoxW32,
                          ...(widget.tableActions ?? []),
                          sizedBoxW8,
                        ],
                      ),
                    ),
                  ),
              ],
            ),
          ),
          ThemedDivider(height: _separator, width: widget.frameWidth),
          SizedBox(
            height: widget.frameHeight - (_topBarHeight + _separator + .5),
            width: widget.frameWidth,
            child: TableContent(
              key: _dataTableContentKey,
              data: _data,
              onFilterChanged: (Map<String, List<FilterField>> filters) {
                setState(() {
                  _filters = filters;
                  widget.onFilterChanged(_filters);
                });
              },
              onSortChanged: widget.onSortChanged,
              uniqueIdField: widget.uniqueIdField,
              filters: _filters,
              selectedListCallback: widget.selectedListCallback,
              columns: widget.columns,
              enableCopyButton: widget.enableCopyButton,
              handleSelectedRow: widget.handleSelectedRow,
            ),
          ),
          const Divider(
            height: .5,
          ),
        ],
      ),
    );
  }
}

class TableSearchBar extends StatelessWidget {
  final Function(String val) changeSearch;
  const TableSearchBar({Key? key, required this.changeSearch})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return TextField(
      onChanged: changeSearch,
      decoration: InputDecoration(
        suffixIcon: ThemedIcons.search(),
      ),
    );
  }
}
