// 🎯 Dart imports:
import 'dart:io';
import 'dart:typed_data';

// 🐦 Flutter imports:
import "package:flutter/material.dart";
import 'package:flutter/foundation.dart';

// 📦 Package imports:
import 'package:chewie/chewie.dart';
import 'package:collection/collection.dart';
import 'package:universal_html/html.dart' as html;
import 'package:verttice_models/verttice_models.dart';
import 'package:video_player/video_player.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class VideoViewerFullPage extends StatefulWidget {
  final String? url;
  final Uint8List? data;
  final String? fileName;
  final bool showAppBar;
  final bool? navigateBackButton;

  VideoViewerFullPage({
    Key? key,
    this.url,
    this.data,
    this.showAppBar = true,
    this.fileName,
    this.navigateBackButton,
  }) : super(key: key) {
    assert(
      url != null || data != null,
      "Must provide url or data",
    );
  }

  @override
  State<VideoViewerFullPage> createState() => _VideoViewerFullPageState();
}

class _VideoViewerFullPageState extends State<VideoViewerFullPage> {
  late VideoPlayerController _videoPlayerController;
  ChewieController? _chewieController;
  String? _url;
  Uint8List? _data;

  @override
  void initState() {
    super.initState();
    _url = widget.url;
    _data = widget.data;
    _initVideoPlayer();
  }

  @override
  void didUpdateWidget(covariant VideoViewerFullPage oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_url != widget.url ||
        !const DeepCollectionEquality().equals(_data, widget.data)) {
      _url = widget.url;
      _data = widget.data;

      _initVideoPlayer();
    }
  }

  @override
  void dispose() {
    super.dispose();
    _videoPlayerController.dispose();
    _chewieController?.dispose();
  }

  Future<void> _initVideoPlayer() async {
    if (kIsWeb && _data != null) {
      final blob = html.Blob([_data]);
      final url = html.Url.createObjectUrlFromBlob(blob);
      _videoPlayerController = VideoPlayerController.network(url);
    } else if (_data != null) {
      _videoPlayerController =
          VideoPlayerController.file(File.fromRawPath(_data!));
    } else {
      _videoPlayerController = VideoPlayerController.network(_url!);
    }
    await _videoPlayerController.initialize();
    _chewieController = ChewieController(
      videoPlayerController: _videoPlayerController,
      looping: true,
    );
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.grey[850],
      appBar: !widget.showAppBar
          ? null
          : ThemedAppBar(
              navigateBackButton: widget.navigateBackButton ?? true,
              actions: [
                DownloadButton(
                  url: widget.url,
                  fileType: VFileType.mp4,
                ),
                sizedBoxH16,
              ],
            ),
      floatingActionButtonLocation: FloatingActionButtonLocation.endTop,
      floatingActionButton: widget.showAppBar
          ? null
          : ThemedCircleAvatar(
              child: DownloadButton(
                url: widget.url,
                fileType: VFileType.mp4,
              ),
            ),
      body: SizedBox(
        child: Builder(builder: (BuildContext context) {
          if (_chewieController == null) {
            return const Center(child: ThemedProgress());
          }

          return Center(
            child: Container(
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(10.0),
              ),
              child: SafeArea(
                child: Chewie(
                  controller: _chewieController!,
                ),
              ),
            ),
          );
        }),
      ),
    );
  }
}
