// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:collection/collection.dart';
import 'package:provider/provider.dart';

// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

class ThemedSearchList<T> extends StatefulWidget {
  final List<Widget>? actions;
  final List<T> items;
  final Widget Function({
    required T item,
    required bool isSelectedItem,
    required ThemedColors themedColors,
  })? itemLabelBuilder;
  final String Function(T item)? itemStringLabelBuilder;
  final Future<void> Function(T) onSelectedItem;
  final bool Function(String search, T item)? searchItemFn;
  final bool Function(T item, T selectedItem)? isSelectedItem;
  final double? rowHeight;

  const ThemedSearchList({
    Key? key,
    required this.items,
    required this.onSelectedItem,
    this.isSelectedItem,
    this.itemLabelBuilder,
    this.itemStringLabelBuilder,
    this.actions,
    this.searchItemFn,
    this.rowHeight,
  }) : super(key: key);

  @override
  State<ThemedSearchList> createState() => _ThemedSearchListState<T>();
}

class _ThemedSearchListState<T> extends State<ThemedSearchList<T>> {
  late List<T> _items;
  T? _selectedItem;
  String _searchText = '';

  @override
  void initState() {
    super.initState();
    _items = widget.items;
  }

  @override
  void didUpdateWidget(covariant ThemedSearchList<T> oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_items, widget.items)) {
      _items = widget.items;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        SizedBox(
          height: 57,
          child: Row(
            children: [
              Expanded(
                child: Padding(
                  padding: const EdgeInsets.symmetric(
                    horizontal: 6.0,
                    vertical: 3,
                  ),
                  child: TextFormFactory(
                    showStandardBorder: false,
                    displayName: 'Search',
                    name: 'search',
                    value: _searchText,
                    onChanged: (name, search) {
                      setState(() {
                        _searchText = search;
                      });
                    },
                  ),
                ),
              ),
              if (widget.actions != null)
                ...widget.actions!
                    .map((e) => Padding(
                          padding: const EdgeInsets.symmetric(horizontal: 4),
                          child: e,
                        ))
                    .toList()
            ],
          ),
        ),
        Expanded(
          child: Builder(builder: (context) {
            final items = _searchText == ''
                ? _items
                : _items.where((item) {
                    return widget.searchItemFn != null
                        ? widget.searchItemFn!(_searchText, item)
                        : item
                            .toString()
                            .toLowerCase()
                            .contains(_searchText.toLowerCase());
                  }).toList();
            return ListView.separated(
              separatorBuilder: (context, index) => divider,
              itemCount: items.length,
              itemBuilder: (context, index) {
                final item = items[index];

                return InkWell(
                  onTap: () async {
                    setState(() {
                      _selectedItem = item;
                    });
                    await widget.onSelectedItem(item);
                  },
                  child: Consumer<VertticeUIState>(builder: (
                    BuildContext context,
                    VertticeUIState value,
                    Widget? _,
                  ) {
                    final isSelectedItem = _selectedItem == null
                        ? false
                        : (widget.isSelectedItem != null
                            ? widget.isSelectedItem!(item, _selectedItem as T)
                            : item == _selectedItem!);
                    return Container(
                      height: widget.rowHeight,
                      padding: const EdgeInsets.all(13),
                      width: double.infinity,
                      color: _selectedItem == null
                          ? Colors.transparent
                          : isSelectedItem
                              ? value.themedColors.accentColor
                              : Colors.transparent,
                      child: Builder(builder: (context) {
                        if (widget.itemLabelBuilder != null) {
                          return Center(
                            child: widget.itemLabelBuilder!(
                              item: item,
                              isSelectedItem: isSelectedItem,
                              themedColors: value.themedColors,
                            ),
                          );
                        }
                        if (widget.itemStringLabelBuilder != null) {
                          return Center(
                            child: Text(
                              widget.itemStringLabelBuilder!(item),
                              style: TextStyle(
                                fontSize: 12,
                                color: isSelectedItem
                                    ? value.themedColors.textColor
                                    : null,
                              ),
                            ),
                          );
                        }
                        return Center(
                          child: Text(
                            item.toString(),
                            style: TextStyle(
                              fontSize: 12,
                              color: isSelectedItem
                                  ? value.themedColors.textColor
                                  : null,
                            ),
                          ),
                        );
                      }),
                    );
                  }),
                );
              },
            );
          }),
        ),
      ],
    );
  }
}
