import 'dart:typed_data';
import 'dart:ui' as ui;

import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:verttice_ui/verttice_ui.dart';

// Future<Uint8List?> createImageSizedFromWidget({
//   required Widget widget,
//   required BuildContext context,
// }) async {
//   Uint8List? image;
//   bool imageArrived = false;
//   final overlay = Overlay.of(context);

//   overlay?.insert(
//     OverlayEntry(
//       builder: (context) => GetImageFromWidget(
//         wait: 5,
//         child: widget,
//         imageCallback: (image) {
//           image = image;
//           imageArrived = true;
//           overlay.rearrange([]);
//         },
//       ),
//     ),
//   );
//   while (!imageArrived && image == null) {
//     await Future.delayed(const Duration(milliseconds: 200));
//   }

//   return image;
// }

Future<Uint8List?> createImageSizedFromWidgetWithLoading({
  required Widget widget,
  required BuildContext context,
}) async {
  final pngBytes = await sizedDialog<Uint8List?>(
    context: context,
    height: 70,
    width: 70,
    builder: (BuildContext context) {
      return Column(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          GetImageFromWidget(
            child: widget,
            imageCallback: (image) {
              return Navigator.of(context).pop(image);
            },
          ),
          centeredLoader,
          sizedBoxH8,
          const ThemedText(
            'Generating Image',
            showThemed: true,
          ),
        ],
      );
    },
  );
  return pngBytes;
}

class GetImageFromWidget extends StatefulWidget {
  final Widget child;
  final int wait;
  final void Function(Uint8List? image) imageCallback;
  const GetImageFromWidget({
    Key? key,
    required this.child,
    required this.imageCallback,
    this.wait = 1,
  }) : super(key: key);

  @override
  State<GetImageFromWidget> createState() => _GetChildImageWidgetState();
}

class _GetChildImageWidgetState extends State<GetImageFromWidget>
    with AfterLayoutMixin<GetImageFromWidget> {
  final GlobalKey _key = GlobalKey();

  Future<void> _getChildImage() async {
    final RenderBox renderChild =
        _key.currentContext!.findRenderObject()! as RenderBox;
    final image = await createImageFromWidget(
      widget.child,
      size: renderChild.size,
    );
    if (image == null) {
      return;
    }
    widget.imageCallback(image);
  }

  @override
  void afterFirstLayout(BuildContext context) async {
    await Future.delayed(Duration(seconds: widget.wait));
    await _getChildImage();
  }

  @override
  Widget build(BuildContext context) {
    return Offstage(
      offstage: true,
      child: Container(
        key: _key,
        child: widget.child,
      ),
    );
  }
}

/// Creates an image from the given widget by first spinning up a element and render tree,
/// then waiting for the given [wait] amount of time and then creating an image via a [RepaintBoundary].
///
/// The final image will be of size [imageSize] and the the widget will be layout, ... with the given [logicalSize].
Future<Uint8List?> createImageFromWidget(
  Widget widget, {
  Duration? wait,
  required Size size,
}) async {
  final RenderRepaintBoundary repaintBoundary = RenderRepaintBoundary();

  final RenderView renderView = RenderView(
    window: WidgetsBinding.instance.window,
    child: RenderPositionedBox(
      alignment: Alignment.center,
      child: repaintBoundary,
    ),
    configuration: ViewConfiguration(
      size: size,
      devicePixelRatio: 1.0,
    ),
  );

  final PipelineOwner pipelineOwner = PipelineOwner();
  final BuildOwner buildOwner = BuildOwner(focusManager: FocusManager());

  pipelineOwner.rootNode = renderView;
  renderView.prepareInitialFrame();

  final RenderObjectToWidgetElement<RenderBox> rootElement =
      RenderObjectToWidgetAdapter<RenderBox>(
    container: repaintBoundary,
    child: MaterialApp(
      showSemanticsDebugger: false,
      debugShowCheckedModeBanner: false,
      home: Scaffold(
        backgroundColor: Colors.transparent,
        body: widget,
      ),
    ),
  ).attachToRenderTree(buildOwner);

  buildOwner.buildScope(rootElement);

  if (wait != null) {
    await Future.delayed(wait);
  }

  buildOwner.buildScope(rootElement);
  buildOwner.finalizeTree();
  pipelineOwner.flushLayout();
  pipelineOwner.flushCompositingBits();
  pipelineOwner.flushPaint();

  final ui.Image image = await repaintBoundary.toImage(
      // pixelRatio: imageSize.width / logicalSize.width,
      );

  final ByteData? byteData = await image.toByteData(
    format: ui.ImageByteFormat.png,
  );

  return byteData?.buffer.asUint8List();
}
